------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a real time simulator mainly designed for educational purpose. 
-- This program provides services to automaticly check temporal constraints 
-- of real time tasks.
--
-- Copyright (C) 2002   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the LIMI Team/EA 2215, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------



with Gtk;
use Gtk;
with Gtk.Main;

with Ada.Io_Exceptions;
use  Ada.Io_Exceptions;

with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;

with Unbounded_Strings;
use  Unbounded_Strings;

with Config;
use Config;

with Project;
use Project;
with Systems;

with Root_Window_Pkg;
use Root_Window_Pkg;

with Gtk.Widget;
use Gtk.Widget;
with Gdk.Font;
use Gdk.Font;
with Gtk.Style;
use Gtk.Style;

with Gnat.Command_Line;
use Gnat.Command_Line;

with  Gnat.Os_Lib;
use Gnat.Os_Lib;

with Text_Io;
use Text_Io;

procedure Cheddar is 


   procedure Copyright is 
   begin
      Put_Line(
         "Cheddar version 1.2, Copyright (C) 2002, F. Singhoff, J. Legrand");
      Put_Line(
         "Cheddar is developped by the LIMI/EA 2215 Team, University of Brest");
      Put_Line("Cheddar comes with ABSOLUTELY NO WARRANTY; ");
      Put_Line(
         "This is free software, and you are welcome to redistribute it");
      Put_Line(
         "under certain conditions; See the GNU General Public License for details.");
   end Copyright;


   procedure Usage is 
   begin
      Put_Line (
         "Cheddar is a real time simulator mainly designed for educational purpose.");
      Put_Line(
         "This program provides services to automaticly check temporal constraints of real time tasks.");
      Copyright;
      New_Line;
      Put_Line(
         "Cheddar home page :  http://beru.univ-brest.fr/~singhoff/cheddar ");
      New_Line;
      New_Line;
      Put_Line("Usage : cheddar [switch] project-name");
      Put_Line("   switch can be :");
      Put_Line("            -u get this help");
      Put_Line(
         "            -l select a language : ""fr"" for franais; ""en"" for english ");
      Put("               Default language is ");
      if(Current_Language = English)
            then
         Put("English.");
      else
         Put("Franais.");
      end if;
      New_Line;
   end Usage;


   My_Style    : Gtk_Style;  
   Global_Font : Gdk_Font;  


begin



   -- New project 
   Do_New;


   -- Get arguments 
   loop
      case Getopt ("u l:") is
         when Ascii.Nul =>
            exit;

         when 'u' =>
            Usage;
            Os_Exit(0);

         when 'l' =>
            if (Parameter /= "fr") and
                  (Parameter /= "en")
                  then
               Put_Line("cheddar.adb : invalid language parameter");
               New_Line;
               Usage;
               Os_Exit(0);
            else
               if Parameter = "fr"
                     then
                  Current_Language:=Francais;
               else
                  Current_Language:=English;
               end if;
            end if;

         when others =>
            Usage;
            Os_Exit(0);
      end case;
   end loop;

   loop
      declare
         S : constant String := Get_Argument (Do_Expansion => True);  

      begin
         exit when S'Length = 0;
         File_Name := File_Name & S;
      end;
   end loop;


   Copyright;
   if(File_Name /= Empty_String)
         then
      Current_Directory:=File_Name;
      Is_New:=False;
      Is_Saved:=True;
      Systems.Read_From_File(Sys, File_Name);
   end if;


   -- Initialize the mmi
   --
   Gtk.Main.Set_Locale;
   Gtk.Main.Init;

   Load (Global_Font,
      "-*-*-*-*-*-*-*-*-*-*-*-*-iso8859-1");
   Gtk_New(My_Style);
   Set_Font(My_Style, Global_Font);
   Set_Default_Style (My_Style);

   Gtk_New (Root_Window);


   -- MMI event loop
   --
   Gtk.Main.Main;



exception
   when Invalid_Switch    =>
      begin
         Put_Line ("cheddar.adb : invalid Switch " & Full_Switch);
         Usage;
      end;
   when Invalid_Parameter =>
      Put_Line ("cheddar.adb : missing parameter for switch " &
         Full_Switch);
   when Ada.Io_Exceptions.Name_Error =>
      Put_Line("cheddar.adb : Can not read project files");
end Cheddar;



