------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a real time simulator mainly designed for educational purpose. 
-- This program provides services to automaticly check temporal constraints 
-- of real time tasks.
--
-- Copyright (C) 2002   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the LIMI Team/EA 2215, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
-----------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Package description :
--
-- You will find here every constants that can be tune by users.
-- Change definitions contained in this package according to your need
------------------------------------------------------------------------------
------------------------------------------------------------------------------






package Config is


   pragma Elaborate_Body;



-- Constant to set the level of debugging information
-- provided by Cheddar.
-- Level 0 = no debugging information
-- Level 1 = ligth debugging information
-- Level 2 = heavy debugging information
--
type debug_levels is range 0 .. 2;

debug_Level : constant debug_levels := 2;




-- Maximum processors, tasks, messages, buffers and resources
-- which can be define in a cheddar simulation
--
   Max_Processors : constant NAtural := 10;  
   Max_Tasks      : constant Natural := 100;  
   Max_Messages   : constant Natural := 50;  
   Max_Buffers    : constant Natural := 30;  
   Max_Resources  : constant Natural := 50;


-- Define the largest scheduling interval that the 
-- simulator can compute
--
   Max_Scheduling_Period : constant Natural := 5000;  


--  Warning : change the two constants
--  after this comment with care : your
--  previously saved Cheddar project could
--  not be loaded succesfully !!!
--
-- Constants above define maximum number of tasks that
-- can be use a shared resource or a buffer
--
max_tasks_for_a_resource  : constant natural := 100;
max_tasks_for_a_buffer   : constant natural := 100; 


-- Definition of priority range. We choose the most current value :
-- 256 priority level (from 0 to 255)
-- Zero is the lowest priority level
---
   type Priority_Range is range 0 .. 255; 
   Low_Priority  : constant Priority_Range := Priority_Range'First;  
   High_Priority : constant Priority_Range := Priority_Range'Last;  



-- Define offset constants : a task can contain at most 3 offsets
-- Warning : current implementation of offset does not allow you
-- to change the constant above. Some points must be fixem in
-- the mmi part berfore (ses widgets to create/update/display tasks) 
--
   Max_Offset : constant Natural := 3;  
   type Offset_Range is new Natural range 1 .. Max_Offset; 


-- Define the language used in the MMI part of Cheddar
-- Default language is "english"
--
   type Languages is 
         (Francais, 
          English); 
   Current_Language : Languages := English;  


end Config;
