------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a real time simulator mainly designed for educational purpose. 
-- This program provides services to automaticly check temporal constraints 
-- of real time tasks.
--
-- Copyright (C) 2002   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the LIMI Team/EA 2215, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------



with Text_Io;
use Text_Io;

package body Resources is


   procedure Initialize (
         A_Resource : in out Generic_Resource ) is 
   begin
      null;
   end Initialize;

   procedure Finalize (
         A_Resource : in out Generic_Resource ) is 
   begin
      null;
   end Finalize;

   procedure Adjust (
         A_Resource : in out Generic_Resource ) is 
   begin
      null;
   end Adjust;



   procedure Initialize (
         A_Resource : in out Np_Resource ) is 
   begin
      A_Resource.Protocol:=No_Protocol;
   end Initialize;

   procedure Finalize (
         A_Resource : in out Np_Resource ) is 
   begin
      null;
   end Finalize;

   procedure Adjust (
         A_Resource : in out Np_Resource ) is 
   begin
      null;
   end Adjust;

   procedure Initialize (
         A_Resource : in out Pcp_Resource ) is 
   begin
      A_Resource.Protocol:=Priority_Ceiling_Protocol;
   end Initialize;

   procedure Finalize (
         A_Resource : in out Pcp_Resource ) is 
   begin
      null;
   end Finalize;

   procedure Adjust (
         A_Resource : in out Pcp_Resource ) is 
   begin
      null;
   end Adjust;


   procedure Initialize (
         A_Resource : in out Pip_Resource ) is 
   begin
      A_Resource.Protocol:=Priority_Inheritance_Protocol;
   end Initialize;

   procedure Finalize (
         A_Resource : in out Pip_Resource ) is 
   begin
      null;
   end Finalize;

   procedure Adjust (
         A_Resource : in out Pip_Resource ) is 
   begin
      null;
   end Adjust;


   function Protocol_To_String (
         A_Protocol : in     Resources_Type ) 
     return Unbounded_String is 

      New_Protocol : Unbounded_String;  

   begin

      --If you want to add a Protocol, Add a conditionnal

      if (A_Protocol = Priority_Ceiling_Protocol) then
         New_Protocol := To_Unbounded_String("PCP");
      elsif (A_Protocol = Priority_Inheritance_Protocol) then
         New_Protocol := To_Unbounded_String("PIP");
      elsif (A_Protocol = No_Protocol ) then
         New_Protocol := To_Unbounded_String("No Protocol");
      else
         New_Protocol := To_Unbounded_String("Error");
      end if;

      return New_Protocol;

   end Protocol_To_String;

   -------------------------------------------------

   procedure Reset (
         A_Blocking_Time : in out Generic_Blocking_Time'Class ) is 
   begin
      A_Blocking_Time.Name:=No_Protocol;
   end Reset;

   procedure Set_Blocking_Time_Type (
         My_Blocking_Time : in out Generic_Blocking_Time'Class; 
         Blocking_Type    : in     Resources_Type               ) is 
   begin
      My_Blocking_Time.Name:=Blocking_Type;
   end Set_Blocking_Time_Type;

   function Get_Blocking_Time_Type (
         My_Blocking_Time : in     Generic_Blocking_Time'Class ) 
     return Unbounded_String is 
      Name : Unbounded_String;  
   begin
      if My_Blocking_Time.Name = No_Protocol
            then
         Name:=To_Unbounded_String("No_protocol");
      else
         if My_Blocking_Time.Name = Priority_Ceiling_Protocol
               then
            Name:= To_Unbounded_String("Priority_Ceiling_Protocol");
         else
            if My_Blocking_Time.Name = Priority_Inheritance_Protocol
                  then
               Name:= To_Unbounded_String("Priority_Inheritance_Protocol");
            end if;
         end if;
      end if;
      return Name;
   end Get_Blocking_Time_Type ;

   procedure Dump (
         My_Resource : in     Resource_Ptr ) is 

   begin

      Put("Name: "& To_String(My_Resource.Name));
      Put("; CPU: "& To_String(My_Resource.Location));
      Put("; Protocol : "& To_String(Protocol_To_String(My_Resource.
               Protocol)));
      Put_Line("; State : "& To_String(To_Unbounded_String(Natural'Image(
                  My_Resource.State))));

      Put_Line("Affected Tasks :");
      Put_Line("Task_Name   Begin   End");
      Dump(My_Resource.Task_Tab);


   end Dump;



   function Copy (
         My_Resource : in     Resource_Ptr ) 
     return Resource_Ptr is 

      New_Np_Resource  : Np_Resource_Ptr;  
      New_Pcp_Resource : Pcp_Resource_Ptr;  
      New_Pip_Resource : Pip_Resource_Ptr;  

      Ptr : Resource_Ptr;  

   begin
      if (My_Resource.Protocol = No_Protocol)
            then
         New_Np_Resource:=new Np_Resource;
         New_Np_Resource.all:=Np_Resource_Ptr(My_Resource).all;
         Ptr:=Resource_Ptr(New_Np_Resource);
      else
         if (My_Resource.Protocol = Priority_Ceiling_Protocol)
               then
            New_Pcp_Resource:=new Pcp_Resource;
            New_Pcp_Resource.all:=Pcp_Resource_Ptr(My_Resource).all;
            Ptr:=Resource_Ptr(New_Pcp_Resource);
         else
            New_Pip_Resource:=new Pip_Resource;
            New_Pip_Resource.all:=Pip_Resource_Ptr(My_Resource).all;
            Ptr:=Resource_Ptr(New_Pip_Resource);
         end if;
      end if;

      return Ptr;
   end Copy;


   procedure Put (
         A : in     Affected_Task_List ) is 
   begin
      Put(A.Task_Begin'Img & "/" & A.Task_End'Img);
   end Put;

   procedure Initialize (
         A :    out Affected_Task_List ) is 
   begin
      A.Task_Begin:=0;
      A.Task_End:=0;
   end Initialize;



   procedure Free (
         A_Resource : in out Resource_Ptr ) is 
   begin
      Put_Line("free resource");
   end Free;


end Resources;



