------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a real time simulator mainly designed for educational purpose. 
-- This program provides services to automaticly check temporal constraints 
-- of real time tasks.
--
-- Copyright (C) 2002   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the LIMI Team/EA 2215, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------






with Task_Set;
use Task_Set;
use Task_Set.Generic_Task_Set;

with Scheduler;


with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;




package Scheduler.Llf is


   type Llf_Scheduler is new Generic_Scheduler with private; 

   type Llf_Tcb is new Tcb with 
      record 
         Dynamic_Deadline : Natural;  
      end record; 
   type Llf_Tcb_Ptr is access all Llf_Tcb'Class; 



   procedure Initialize (
         A_Scheduler : in out Llf_Scheduler ); 

   procedure Finalize (
         A_Scheduler : in out Llf_Scheduler ); 

   procedure Adjust (
         A_Scheduler : in out Llf_Scheduler ); 


   function Build_Tcb (
         My_Scheduler : in     Llf_Scheduler; 
         A_Task       :        Task_Ptr       ) 
     return Tcb_Ptr; 

   procedure Check_Before_Scheduling (
         My_Scheduler   : in     Llf_Scheduler;   
         My_Tasks       : in     Tasks_Set;       
         Processor_Name : in     Unbounded_String ); 

   procedure Do_Election (
         My_Scheduler      : in out Llf_Scheduler;          
         Si                : in out Scheduling_Information; 
         Processor_Name    : in     Unbounded_String;       
         At_Time           :        Natural;                
         My_Dependencies   : in     Tasks_Dependencies;     
         With_Offsets      :        Boolean;                
         With_Precedencies :        Boolean;                
         With_Resources    :        Boolean;                
         Elected           : in out Tasks_Range;            
         No_Task           : in out Boolean                 ); 



   procedure Dispatched_Response_Time (
         My_Scheduler   : in     Llf_Scheduler;    
         My_Tasks       : in out Tasks_Set;        
         Processor_Name : in     Unbounded_String; 
         Msg            : in out Unbounded_String; 
         Response_Time  :    out Response_Table    ); 

   procedure Dispatched_Bound_On_Processor_Utilization (
         My_Scheduler   : in     Llf_Scheduler;    
         My_Tasks       : in     Tasks_Set;        
         Processor_Name : in     Unbounded_String; 
         Result         : in out Double;           
         Msg            : in out Unbounded_String  ); 



   procedure Specific_Scheduler_Initialization (
         My_Scheduler   : in out Llf_Scheduler;          
         Si             : in out Scheduling_Information; 
         Processor_Name : in     Unbounded_String;       
         My_Tasks       : in out Tasks_Set;              
         My_Resources   : in out Resources_Set           ); 




private


   type Llf_Scheduler is new Generic_Scheduler with null record; 


end Scheduler.Llf ;


