------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a real time simulator mainly designed for educational purpose.
-- This program provides services to automaticly check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the LIMI Team/EA 2215, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------



with Text_Io;
use Text_Io;

with Io_Definitions;
use Io_Definitions;

with Network;
use Network;

with Ada.Io_Exceptions;
use  Ada.Io_Exceptions;


package body Systems is


   procedure Read_From_File (
         A_System  : in out System;          
         File_Name : in     Unbounded_String ) is 

   begin

      Read_From_File(A_System.Processors, File_Name & ".proc" );


      begin
         Read_From_File(A_System.Tasks, File_Name & ".tsk" );
      exception
         when Ada.Io_Exceptions.Name_Error =>
            null;
      end;

      begin
         Read_From_File(A_System.Messages, File_Name & ".msg" );

      exception
         when Ada.Io_Exceptions.Name_Error =>
            null;
      end;

      begin
         Read_From_File(A_System.Buffers, File_Name & ".buff" );
      exception
         when Ada.Io_Exceptions.Name_Error =>
            null;
      end;
      begin
         Read_From_File(A_System.Resources, File_Name & ".res" );
      exception
         when Ada.Io_Exceptions.Name_Error =>
            null;
      end;
      begin
         Read_From_File(A_System.Dependencies,
            A_System.Tasks,
            A_System.Messages,
            A_System.Buffers,
            File_Name & ".dep");
      exception
         when Ada.Io_Exceptions.Name_Error =>
            null;
      end;
   end Read_From_File;


   procedure Write_To_File (
         A_System  : in     System;          
         File_Name : in     Unbounded_String ) is 

   begin


      Write_To_File(A_System.Processors, File_Name & ".proc",
         Save_In_New_File );

      Write_To_File(A_System.Tasks, File_Name & ".tsk",
         Save_In_New_File );

      Write_To_File(A_System.Messages, File_Name & ".msg",
         Save_In_New_File );

      Write_To_File(A_System.Buffers, File_Name & ".buff",
         Save_In_New_File );

      Write_To_File(A_System.Resources, File_Name & ".res",
         Save_In_New_File );

      Write_To_File(A_System.Dependencies, File_Name & ".dep",
         Save_In_New_File );

   end Write_To_File;


   procedure Dump (
         A_System : in     System ) is 
   begin
      -- Dump(A_System.Network.all);


      Put_Line("***** LIST OF PROCESSORS *****");
      Dump(A_System.Processors);
      New_Line;
      New_Line;
      New_Line;
      Put_Line("***** LIST OF TASKS *****");
      Dump(A_System.Tasks);
      New_Line;
      New_Line;
      New_Line;
      Put_Line("***** LIST OF RESSOURCES *****");
      Dump(A_System.Resources);
      New_Line;
      New_Line;
      New_Line;
      Put_Line("***** LIST OF BUFFERS *****");
      Dump(A_System.Buffers);
      New_Line;
      New_Line;
      New_Line;
      Put_Line("***** LIST OF MESSAGES *****");
      Dump(A_System.Messages);
      New_Line;
      New_Line;
      New_Line;
      Put_Line("***** LIST OF DEPENCENCIES *****");
      Dump(A_System.Dependencies);

   end Dump;



   procedure Reset (
         A_System : in out System ) is 
   begin
      A_System.Network:= null;
      Reset(A_System.Tasks);
      Reset(A_System.Messages);
      Reset(A_System.Resources);
      Reset(A_System.Dependencies);
      Reset(A_System.Buffers);
      Reset(A_System.Processors);
   end Reset;



end Systems;


