------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a real time simulator mainly designed for educational purpose.
-- This program provides services to automaticly check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the LIMI Team/EA 2215, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------




with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;

with Message_Set;
use Message_Set;
use Message_Set.Generic_Message_Set;

with Buffer_Set;
use Buffer_Set;
use Buffer_Set.Generic_Buffer_Set;

with Task_Set;
use Task_Set;
use Task_Set.Generic_Task_Set;

with Task_Dependencies;
use Task_Dependencies;
use Task_Dependencies.Half_Dep_Set;

with Processor_Set;
use Processor_Set;
use Processor_Set.Generic_Processor_Set;

with Resource_Set;
use Resource_Set;
use Resource_Set.Generic_Resource_Set;

with Network;
use Network;


package Systems is


   type System is tagged 
      record 
         Network      : Network_Ptr;  
         Tasks        : Tasks_Set;  
         Resources    : Resources_Set;  
         Messages     : Messages_Set;  
         Dependencies : Tasks_Dependencies;  
         Processors   : Processors_Set;  
         Buffers      : Buffers_Set;  
      end record; 

   procedure Reset (
         A_System : in out System ); 

   procedure Dump (
         A_System : in     System ); 

   procedure Read_From_File (
         A_System  : in out System;          
         File_Name : in     Unbounded_String ); 

   procedure Write_To_File (
         A_System  : in     System;          
         File_Name : in     Unbounded_String ); 

end Systems;








