------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a real time simulator mainly designed for educational purpose.
-- This program provides services to automaticly check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the LIMI Team/EA 2215, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------



with Offsets;
use Offsets;

with Ada.Finalization;

with Config;
use  Config;

with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;




package Tasks is

   Compute_Aborded : exception;  

   type Tasks_Type is 
         (Periodic_Type,  
          Aperiodic_Type, 
          Generic_Type); 


   type Policies is 
         (Sched_Fifo,  
          Sched_Rr,    
          Sched_Others); 


   --------------------------------------------------

   type Generic_Task is abstract new Ada.Finalization.Controlled with 
      record 
         Name          : Unbounded_String := To_Unbounded_String ("");  
         Task_Type     : Tasks_Type;  
         Location      : Unbounded_String := To_Unbounded_String ("");  
         Capacity      : Natural          := 0;  
         Deadline      : Natural          := 0;  
         Start_Time    : Natural          := 0;  
         Priority      : Priority_Range   := Priority_Range'First;  
         Blocking_Time : Natural          := 0;  
         Policy        : Policies;  
         Offset        : Offset_Table;  
      end record; 

   type Task_Ptr is access all Generic_Task'Class; 



   procedure Initialize (
         A_Task : in out Generic_Task ); 
   procedure Finalize (
         A_Task : in out Generic_Task ); 
   procedure Adjust (
         A_Task : in out Generic_Task ); 


   procedure Free (
         My_Task : in out Task_Ptr ); 
   pragma Import (C, Free, "__gnat_free");


   procedure Dump (
         My_Task : in     Generic_Task'Class ); 

   procedure Dump (
         My_Task : in     Task_Ptr ); 

   procedure Dispatched_Dump (
         My_Task : in     Generic_Task ) is 
   abstract;

   procedure Put_Name (
         A_Task : in     Task_Ptr ); 


   function Copy (
         My_Task : in     Task_Ptr ) 
     return Task_Ptr; 



   --------------------------------------------------

   type Periodic_Task is new Generic_Task with 
      record 
         Period : Natural := 0;  
         Jitter : Natural := 0;  
      end record; 

   type Periodic_Task_Ptr is access all Periodic_Task'Class; 


   procedure Initialize (
         A_Task : in out Periodic_Task ); 
   procedure Finalize (
         A_Task : in out Periodic_Task ); 
   procedure Adjust (
         A_Task : in out Periodic_Task ); 

   procedure Dispatched_Dump (
         My_Task : in     Periodic_Task ); 


   --------------------------------------------------


   type Aperiodic_Task is new Generic_Task with null record; 

   type Aperiodic_Task_Ptr is access all Aperiodic_Task'Class; 


   procedure Initialize (
         A_Task : in out Aperiodic_Task ); 
   procedure Finalize (
         A_Task : in out Aperiodic_Task ); 
   procedure Adjust (
         A_Task : in out Aperiodic_Task ); 

   procedure Dispatched_Dump (
         My_Task : in     Aperiodic_Task ); 







end Tasks;
