------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a real time simulator mainly designed for educational purpose.
-- This program provides services to automaticly check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the LIMI Team/EA 2215
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------



with Config;
use Config;
--with Project;
--use Project;
-- with Double_Util;
-- use Double_Util;
with Translate;
use Translate;
--with Tasks;
--use Tasks;

with Task_Set;
use Task_Set;
--use Task_Set.Generic_Task_Set;

with Buffers;
use Buffers;

with Buffer_Set;
use Buffer_Set;
use Buffer_Set.Generic_Buffer_Set;
-- with Ada.Numerics.Aux;
-- use Ada.Numerics.Aux;
with Unbounded_Strings;
use Unbounded_Strings;

package body Call_Buffer_Logic is

   procedure Compute_Buffer_Bound (
         My_Buffers     : in out Buffers_Set;      
         My_Tasks       : in     Tasks_Set;        
         Processor_Name : in     Unbounded_String; 
         Message        : in out Unbounded_String  ) is 


      A_Buffer    : Buffer_Ptr;  
      Bnd         : Natural;  
      My_Iterator : Iterator;  
   begin

      Reset_Iterator(My_Iterator);

      Message := Message & Lb_Bound_Buffer(Current_Language) &
         Unbounded_Lf;

      loop
         Current_Element(My_Buffers, A_Buffer, My_Iterator);
         if (A_Buffer.Location = Processor_Name) then
            if (Natural(A_Buffer.Roles.Nb_Entries) >= 1)
                  then
               begin

                  if (Is_Cons_Prod_Harmonic(My_Tasks,A_Buffer.Roles) =
                        True)
                        then -- hamonic producers/consumers
                     Bnd := Get_Bound_Harmonic(A_Buffer.Roles, My_Tasks,
                        Processor_Name);

                     Message:= Message & To_Unbounded_String("    ") &
                        A_Buffer.Name &
                        To_Unbounded_String(" => ") &
                        Bnd'Img & To_Unbounded_String(" (") & Lb_See(
                        Current_Language) &
                        To_Unbounded_String("[4], ") & Lb_Equation(
                        Current_Language) &
                        To_Unbounded_String("13). ") & Unbounded_Lf;

                  else
                     Bnd := Get_Bound(A_Buffer.Roles, My_Tasks,
                        Processor_Name);

                     Message:= Message &  To_Unbounded_String("    ") &
                        A_Buffer.Name
                        & To_Unbounded_String(" => ") & Bnd'Img &
                        To_Unbounded_String(" (") &
                        Lb_See(Current_Language) & To_Unbounded_String(
                        "[4],  ") &
                        Lb_Equation(Current_Language) &
                        To_Unbounded_String("14). ")
                        & Unbounded_Lf;
                  end if;

               exception
                  when Task_Set.Task_Must_Be_Periodic=>
                     Message:=Message & Lb_Compute_Error_17(
                        Current_Language) &
                        Unbounded_Lf;
                  when Buffers.Flow_Constraint_Not_Respected =>
                     Message:=Message & Lb_Compute_Error_16(
                        Current_Language) &
                        Unbounded_Lf;
               end;
            else
               Message:= Message &  To_Unbounded_String("    ") & A_Buffer.
                  Name
                  & To_Unbounded_String(" => NA");
            end if;
         end if;
         exit when Is_Last_Element(My_Buffers, My_Iterator) = True;
         Next_Element(My_Buffers, My_Iterator);
      end loop;

   end Compute_Buffer_Bound;

end Call_Buffer_Logic;




