------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a real time simulator mainly designed for educational purpose. 
-- This program provides services to automaticly check temporal constraints 
-- of real time tasks.
--
-- Copyright (C) 2002   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the LIMI Team/EA 2215, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------



with Gtk;
use Gtk;
with Gdk.Types;
use Gdk.Types;
with Gtk.Widget;
use Gtk.Widget;
with Gtk.Enums;
use Gtk.Enums;

with Callbacks_Cheddar;
use Callbacks_Cheddar;
with Options_Pkg.Callbacks;
use Options_Pkg.Callbacks;

with Config;
use Config;
with Translate;
use Translate;
with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;

with Project;
use Project;


package body Options_Pkg is

   procedure Gtk_New (
         Options :    out Options_Access ) is 

   begin
      Options := new Options_Record;
      Options_Pkg.Initialize (Options);
   end Gtk_New;

   procedure Initialize (
         Options : access Options_Record'Class ) is 

   begin

      Gtk.Window.Initialize (Options, Window_Toplevel);
      Set_Title (Options, To_String(Lb_Scheduling_Options(
               Current_Language)));
      Set_Policy (Options, True, True, True);
      Set_Position (Options, Win_Pos_Center_Always);
      Set_Modal (Options, True);
      Set_Extension_Events (Options, Extension_Events_All);
      Set_Flags (Options, Can_Default);
      Grab_Focus (Options);


      Gtk_New (Options.Vbox, 4, 2, True);
      Set_Border_Width (Options.Vbox, 14);
      Set_Row_Spacings (Options.Vbox, 8);
      Set_Col_Spacings (Options.Vbox, 0);
      Add (Options, Options.Vbox);

      Gtk_New (Options.Label, To_String(Lb_Schedule_According_To(
               Current_Language)));
      Set_Alignment (Options.Label, 0.5, 0.5);
      Set_Padding (Options.Label, 0, 0);
      Set_Justify (Options.Label, Justify_Left);
      Set_Line_Wrap (Options.Label, False);
      Attach (Options.Vbox, Options.Label, 0,
         1, 0, 1, 0, 0, 0, 0);


      Gtk_New (Options.With_Offsets, To_String(Lb_Offsets(
               Current_Language)));
      if Schedule_With_Offsets = True
            then
         Set_Active(Options.With_Offsets, True);
      else
         Set_Active(Options.With_Offsets, False);
      end if;
      Attach (Options.Vbox, Options.With_Offsets,
         1, 2, 1, 2, Expand or Fill, 0, 0, 0);

      Gtk_New (Options.With_Precedencies, To_String(Lb_Precedencies(
               Current_Language)));
      if Schedule_With_Precedencies = True
            then
         Set_Active(Options.With_Precedencies, True);
      else
         Set_Active(Options.With_Precedencies, False);
      end if;
      Attach (Options.Vbox, Options.With_Precedencies,
         1, 2, 2, 3, Expand or Fill, 0, 0, 0);

      Gtk_New (Options.With_Resources, To_String(Lb_Resources(
               Current_Language)));
      if Schedule_With_Resources = True
            then
         Set_Active(Options.With_Resources, True);
      else
         Set_Active(Options.With_Resources, False);
      end if;
      Attach (Options.Vbox, Options.With_Resources,
         1, 2, 3, 4, Expand or Fill, 0, 0, 0);



      Gtk_New_Hseparator (Options.Hseparator);
      Attach (Options.Vbox, Options.
         Hseparator, 0, 2, 5, 6,
         Fill, Expand or Fill,
         0, 0);

      Gtk_New_Hbox (Options.Hbox, True, 0);
      Set_Border_Width (Options.Hbox, 6);
      Attach (Options.Vbox, Options.Hbox, 0, 2,
         6, 7,
         Fill, Expand or Fill,
         0, 0);

      Gtk_New (Options.Ok_Button, "Ok");
      Pack_Start (Options.Hbox, Options.
         Ok_Button, True, True, 0);
      Button_Callback.Connect
         (Options.Ok_Button, "clicked",
         Button_Callback.To_Marshaller (On_Ok_Pressed'access));

      Gtk_New_Vseparator (Options.Vseparator);
      Set_Sensitive (Options.Vseparator, False);
      Pack_Start (Options.Hbox, Options.
         Vseparator,  True, True, 0);

      Gtk_New (Options.Cancel_Button, To_String(Lb_Cancel(
               Current_Language)));
      Pack_Start (Options.Hbox, Options.
         Cancel_Button, True, True, 0);
      Button_Callback.Connect
         (Options.Cancel_Button, "clicked",
         Button_Callback.To_Marshaller (On_Cancel_Pressed'access));


   end Initialize;

end Options_Pkg;
