
------------------------------------------------------------------------------
-- Cheddar is a real time simulator mainly designed for educational purpose.
-- This program provides services to automaticly check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the LIMI Team/EA 2215, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------



with Config;
use Config;

with Translate;
use Translate;

with Project;
use Project;

with References;
with Manual;

with Processors;
use Processors;
with Processor_Set;
use Processor_Set;
use Processor_Set.Generic_Processor_Set;

with Resources;
use Resources;
with Resource_Set;
use Resource_Set;
use Resource_Set.Generic_Resource_Set;

with Tasks;
use Tasks;
with Task_Set;
use Task_Set;
use Task_Set.Generic_Task_Set;

with Buffers;
use Buffers;
with Buffer_Set;
use Buffer_Set;
use Buffer_Set.Generic_Buffer_Set;

with Dependencies;
use Dependencies;
with Task_Dependencies;
use Task_Dependencies;

with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;

with Unbounded_Strings;
use Unbounded_Strings;

with Messages;
use Messages;
with Message_Set;
use Message_Set;
use Message_Set.Generic_Message_Set;

with Display_Resources_Set_Pkg;
use Display_Resources_Set_Pkg;
with Display_Messages_Set_Pkg;
use Display_Messages_Set_Pkg;

with Scheduler;
use Scheduler;

with Call_Scheduling_Logic;
use Call_Scheduling_Logic;
with Call_Buffer_Logic;
use Call_Buffer_Logic;
with Call_Blocking_Time_Logic;
use Call_Blocking_Time_Logic;



with Glib;
use Glib;
with Gtk.Widget;
use Gtk.Widget;


with Ihm_Tools;
use Ihm_Tools;

with Tasks_Parameters_Pkg;
use Tasks_Parameters_Pkg;
with Processors_Parameters_Pkg;
use Processors_Parameters_Pkg;
with Display_Tasks_Set_Pkg;
use Display_Tasks_Set_Pkg;
with Display_Buffers_Set_Pkg;
use Display_Buffers_Set_Pkg;
with Update_Tasks_Set_Pkg;
use Update_Tasks_Set_Pkg;
with Display_Processors_Set_Pkg;
use Display_Processors_Set_Pkg;
with Update_Processors_Set_Pkg;
use Update_Processors_Set_Pkg;
with Update_Resources_Set_Pkg;
use Update_Resources_Set_Pkg;
with Update_Resources_Set_Pkg.Callbacks;
use Update_Resources_Set_Pkg.Callbacks;
with Update_Buffers_Set_Pkg;
use Update_Buffers_Set_Pkg;
with Update_Messages_Set_Pkg;
use Update_Messages_Set_Pkg;
with Draw_Scheduling_Pkg;
use Draw_Scheduling_Pkg;
with Scrolling_Draw_Buffer_Pkg;
use Scrolling_Draw_Buffer_Pkg;
with Scrolling_Text_Pkg;
use Scrolling_Text_Pkg;
with Question_Pkg;
use Question_Pkg;
with Unbounded_Strings;
use Unbounded_Strings;
with Resources_Parameters_Pkg;
use Resources_Parameters_Pkg;
with Messages_Parameters_Pkg;
use Messages_Parameters_Pkg;
with Buffers_Parameters_Pkg;
use Buffers_Parameters_Pkg;
with Graph_Window_Pkg;
use Graph_Window_Pkg;
with Options_Pkg;
use Options_Pkg;
with Duplicate_Pkg;
use Duplicate_Pkg;

with Multiprocessor_Services;
use Multiprocessor_Services;
use Multiprocessor_Services.Scheduling_Result_Per_Processor;



with Text_Io;
use Text_Io;

package body Root_Window_Pkg.Callbacks is

   use Gtk.Arguments;


   --------------------------------------------------
   --    Tools procedures
   --
   --    These procedure are called by tools menu
   --    and toolbar callbacks
   --------------------------------------------------

   procedure Check_Scheduling is 

      A_Processor : Processor_Ptr;  
      My_Iterator : Processors_Iterator;  
      Msg         : Unbounded_String;  

   begin

      if (Get_Number_Of_Elements(Sys.Tasks)=0)
            then
         Show_Message_Box(Lb_Define_Tasks_Before(Current_Language));
         return;
      end if;

      Write_Text(Root_Window, Unbounded_Lf & Unbounded_Lf & Unbounded_Lf);

      Reset_Iterator(My_Iterator);

      loop
         Current_Element(Sys.Processors, A_Processor, My_Iterator);


         if (Get_Number_Of_Task_From_Processor(Sys.Tasks, A_Processor.
                  Name)/=0)
               then

            Write_Title(Root_Window, Lb_Processor(Current_Language)  &
               A_Processor.Name  & To_Unbounded_String(" : ") &
               Unbounded_Lf & Unbounded_Lf);

            Msg:=To_Unbounded_String("");
            Compute_Scheduling_Period(A_Processor, Msg);
            Write_Text(Root_Window, Msg);

            Msg:=To_Unbounded_String("");
            Processor_Utilization(A_Processor, Msg);
            Write_Text(Root_Window, Msg);

            Msg:=To_Unbounded_String("");
            Compute_Response_Time(A_Processor, Msg);
            Write_Text(Root_Window, Msg);

            Write_Text(Root_Window, Unbounded_Lf & Unbounded_Lf);

         end if;

         exit when Is_Last_Element(Sys.Processors, My_Iterator) = True;

         Next_Element(Sys.Processors, My_Iterator);

      end loop;

   end Check_Scheduling;




   function Draw_Scheduling_Callback (
         Value : Unbounded_String ) 
     return Boolean is 

      Msg         : Unbounded_String;  
      Last_Period : Natural          := Config.Max_Scheduling_Period - 1;
         
      Max_Period  : Natural          := 0;  
      Ok          : Boolean;  

   begin


      To_Natural(Value,Max_Period,Ok);
      if not Ok
            then
         Show_Message_Box(Lb_Parameter(Current_Language) &
            Lb_Must_Be_Numeric(Current_Language));
         return False;
      end if;

      if(Max_Period>=Last_Period)
            then
         Show_Message_Box(Lb_Parameter(Current_Language) & Lb_Must_Be(
               Current_Language) & To_Unbounded_String(" < ")
            & Last_Period'Img);
         return False;
      end if;

      if(Max_Period=0)
            then
         Show_Message_Box(Lb_Parameter(Current_Language) &
            Lb_Must_Be(Current_Language) & To_Unbounded_String(" > 0"));
         return False;
      end if;



      Msg:=To_Unbounded_String("");
      Build_And_Check_Scheduling(Root_Window,  Max_Period);


      Draw_Task(Draw_Scheduling,Sys.Tasks,Max_Period,
         Call_Scheduling_Logic.Sched);


      return True;

   end Draw_Scheduling_Callback;



   procedure Schedule_And_Check_Temporal_Constraints is 


      A_Task      : Task_Ptr;  
      Iterator1   : Tasks_Iterator;  
      A_Processor : Processor_Ptr;  
      Iterator2   : Processors_Iterator;  
      Max_Period  : Natural             := 0;  
      Period      : Natural             := 0;  
      Last_Period : Natural             := Config.Max_Scheduling_Period -
        1;  

   begin

      if (Get_Number_Of_Elements(Sys.Tasks)=0)
            then
         Show_Message_Box(
            Lb_Define_Tasks_Before(Current_Language));
         return;
      end if;


      -- First of all : get the largest period of all
      -- the defined tasks
      --
      Reset_Iterator(Iterator1);
      loop

         Current_Element(Sys.Tasks, A_Task, Iterator1);
         if (A_Task.Task_Type = Periodic_Type)
               then
            Max_Period:=Natural'Max(Max_Period,
               Periodic_Task_Ptr(A_Task).Period);
         else
            Max_Period:=Natural'Max(Max_Period,
               A_Task.Deadline);
         end if;

         exit when Is_Last_Element(Sys.Tasks, Iterator1) = True;

         Next_Element(Sys.Tasks, Iterator1);

      end loop;




      -- Then, compute LCM 
      --
      Reset_Iterator(Iterator2);

      loop


         Current_Element(Sys.Processors, A_Processor, Iterator2);

         if (Get_Number_Of_Task_From_Processor(Sys.Tasks, A_Processor.
                  Name)/=0)
               then

            begin
               Period:=Scheduling_Period(A_Processor.Scheduler.all, Sys.
                  Tasks, A_Processor.Name);
               if Period> Max_Period
                     then
                  Max_Period:=Period;
               end if;

            exception
               when Task_Set.Task_Must_Be_Periodic =>
                  null;
               when Task_Set.Task_Model_Error =>
                  null;

            end;
         end if;

         exit when Is_Last_Element(Sys.Processors, Iterator2) = True;

         Next_Element(Sys.Processors, Iterator2);


      end loop;


      if(Max_Period>=Last_Period)
            then
         Max_Period:=Last_Period;
      end if;

      Gtk_New(Question, Lb_Draw(Current_Language), Lb_Schedule_From_To(
            Current_Language),
         To_Unbounded_String(Max_Period'Img),
         Draw_Scheduling_Callback'access);
      Show_All(Question);

   end Schedule_And_Check_Temporal_Constraints;


   procedure Check_Buffer is 

      A_Processor : Processor_Ptr;  
      My_Iterator : Processors_Iterator;  
      Msg         : Unbounded_String;  

   begin

      if (Get_Number_Of_Elements(Sys.Buffers)=0)
            then
         Show_Message_Box(Lb_Define_Buffers_Before(Current_Language));
         return;
      end if;

      Write_Text(Root_Window, Unbounded_Lf & Unbounded_Lf & Unbounded_Lf);

      Reset_Iterator(My_Iterator);

      loop
         Current_Element(Sys.Processors, A_Processor, My_Iterator);


         if (Get_Number_Of_Buffer_From_Processor (Sys.Buffers, A_Processor.
                  Name) /= 0 )
               then
            Write_Title(Root_Window,Lb_Buffer(Current_Language) &
               To_Unbounded_String(" (")
               & Lb_Processor(Current_Language)  &
               A_Processor.Name & To_Unbounded_String(")") &
               To_Unbounded_String(" : ")
               & Unbounded_Lf & Unbounded_Lf);

            Msg:=To_Unbounded_String("");
            Compute_Buffer_Bound ( Sys.Buffers,
               Sys.Tasks,
               A_Processor.Name,
               Msg);
            Write_Text(Root_Window, Msg);


            Write_Text(Root_Window, Unbounded_Lf & Unbounded_Lf);

         end if;

         -- next processor
         exit when Is_Last_Element(Sys.Processors, My_Iterator) = True;
         Next_Element(Sys.Processors, My_Iterator);
      end loop;

   end Check_Buffer;



   procedure On_Draw_Buffer_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 


      My_Size     : Buffer_Size_Table;  
      My_Iterator : Buffers_Iterator;  
      A_Buffer    : Buffer_Ptr;  

   begin

      if (Get_Number_Of_Elements(Sys.Buffers)=0)
            then
         Show_Message_Box(
            Lb_Define_Buffers_Before(Current_Language));
      else

         -- Check if scheduling is computed or not
         --
         if Call_Scheduling_Logic.Sched.Nb_Entries=0
               then
            Show_Message_Box(Lb_Schedule_Before(Current_Language));
         end if;


         -- For each buffer, compute and draw its size from
         -- 0 to the base period
         --

         loop

            Current_Element(Sys.Buffers, A_Buffer, My_Iterator);


            for I in 0..Call_Scheduling_Logic.Sched.Nb_Entries-1 loop

               if Call_Scheduling_Logic.Sched.Entries(I).Data.Processor/=null then
                  if Call_Scheduling_Logic.Sched.Entries(I).Data.
                        Processor.Name
                        = A_Buffer.Location
                        then

                     Initialize(My_Size);
                     Get_Buffer_Size(Sched.Entries(I).Data.Result,
                        A_Buffer.Roles, Sys.Tasks, My_Size);

                     Gtk_New(Scrolling_Draw_Buffer,To_String(A_Buffer.
                           Name));
                     Show_All(Scrolling_Draw_Buffer);
                     Clear(Scrolling_Draw_Buffer);
                     Draw(Scrolling_Draw_Buffer, A_Buffer, My_Size,
                        Scheduling_Base_Period,
                        Sched.Entries(I).Data.Result.Scheduled.Nb_Entries);

                     exit;
                  end if;
               end if;

            end loop;


            exit when Is_Last_Element(Sys.Buffers, My_Iterator) =
               True;
            Next_Element(Sys.Buffers, My_Iterator);

         end loop;


      end if;

   end On_Draw_Buffer_Activate;


   procedure Clear_Work_Space is 
      Size        : Guint;  
      I_Dont_Care : Boolean;  
   begin
      Size:=Get_Length(Root_Window.Text);
      Set_Point(Root_Window.Text,0);
      I_Dont_Care:=Forward_Delete(Root_Window.Text,Size);
      Clear_Task(Draw_Scheduling);
   end Clear_Work_Space;


   ---------------------------------------------------
   --    Delete functions : called by delete callbacks
   ---------------------------------------------------

   function Delete_Resource_Callback (
         Value : Unbounded_String ) 
     return Boolean is 

      My_Iterator : Resources_Iterator;  
      A_Resource  : Resource_Ptr;  
   begin

      loop

         Current_Element(Sys.Resources, A_Resource, My_Iterator);

         if A_Resource.Name = Value
               then

            Delete(Sys.Resources,A_Resource);

            Is_New:=False;
            Is_Saved:=False;

            return True;
         end if;

         exit when Is_Last_Element(Sys.Resources, My_Iterator) = True;
         Next_Element(Sys.Resources, My_Iterator);

      end loop;


      Show_Message_Box(Lb_Resource_Not_Found(Current_Language));
      return False;

   end Delete_Resource_Callback;


   function Delete_Buffer_Callback (
         Value : Unbounded_String ) 
     return Boolean is 

      My_Iterator : Buffers_Iterator;  
      A_Buffer    : Buffer_Ptr;  
   begin

      loop

         Current_Element(Sys.Buffers, A_Buffer, My_Iterator);

         if A_Buffer.Name = Value
               then

            Delete_All_Dependencies(Sys.Dependencies, Buffer_Dependency,
               A_Buffer.Name);

            Delete(Sys.Buffers,A_Buffer);

            Is_New:=False;
            Is_Saved:=False;

            Rebuild_Canvas(Graph_Window);
            return True;
         end if;

         exit when Is_Last_Element(Sys.Buffers, My_Iterator) = True;
         Next_Element(Sys.Buffers, My_Iterator);

      end loop;


      Show_Message_Box(Lb_Buffer_Not_Found(Current_Language));
      return False;

   end Delete_Buffer_Callback;


   function Delete_Message_Callback (
         Value : Unbounded_String ) 
     return Boolean is 

      My_Iterator : Messages_Iterator;  
      A_Message   : Message_Ptr;  
   begin

      loop

         Current_Element(Sys.Messages, A_Message, My_Iterator);

         if A_Message.Name = Value
               then

            Delete_All_Dependencies(Sys.Dependencies,
               Communication_Dependency,
               A_Message.Name);

            Delete(Sys.Messages,A_Message);

            Is_New:=False;
            Is_Saved:=False;

            Rebuild_Canvas(Graph_Window);
            return True;
         end if;

         exit when Is_Last_Element(Sys.Messages, My_Iterator) = True;
         Next_Element(Sys.Messages, My_Iterator);

      end loop;


      Show_Message_Box(Lb_Buffer_Not_Found(Current_Language));
      return False;

   end Delete_Message_Callback;


   function Delete_Task_Callback (
         Value : Unbounded_String ) 
     return Boolean is 

      My_Iterator : Tasks_Iterator;  
      A_Task      : Task_Ptr;  
   begin

      loop
         Current_Element(Sys.Tasks, A_Task, My_Iterator);

         if A_Task.Name = Value
               then

            Delete_Task(Sys.Resources, A_Task.Name);
            Delete_Task(Sys.Buffers, A_Task.Name);
            Delete_All_Dependencies(Sys.Dependencies,
               Precedence_Dependency, A_Task.Name);
            Delete(Sys.Tasks,A_Task);

            Is_New:=False;
            Is_Saved:=False;

            Rebuild_Canvas(Graph_Window);
            return True;
         end if;

         exit when Is_Last_Element(Sys.Tasks, My_Iterator) = True;
         Next_Element(Sys.Tasks, My_Iterator);

      end loop;


      Show_Message_Box(Lb_Task_Not_Found(Current_Language));
      return False;

   end Delete_Task_Callback;

   function Delete_Processor_Callback (
         Value : Unbounded_String ) 
     return Boolean is 

      Iterator1   : Processors_Iterator;  
      A_Processor : Processor_Ptr;  

   begin

      Reset_Iterator(Iterator1);

      loop

         Current_Element(Sys.Processors, A_Processor, Iterator1);

         if A_Processor.Name = Value
               then

            Delete_Processors_Dependencies(Sys.Dependencies, A_Processor.
               Name);
            Delete_Processor(Sys.Buffers, A_Processor.Name);
            Delete_Processor(Sys.Resources, A_Processor.Name);
            Delete_Processor(Sys.Tasks, A_Processor.Name);
            Delete(Sys.Processors,A_Processor);

            Is_New:=False;
            Is_Saved:=False;

            Rebuild_Canvas(Graph_Window);
            return True;
         end if;

         exit when Is_Last_Element(Sys.Processors, Iterator1) = True;
         Next_Element(Sys.Processors, Iterator1);

      end loop;


      Show_Message_Box(Lb_Processor_Not_Found(Current_Language));
      return False;

   end Delete_Processor_Callback;


   --------------------------------------------------
   --    Others menu callbacks
   --------------------------------------------------

   function On_Root_Window_Delete_Event (
         Object : access Gtk_Widget_Record'Class; 
         Params :        Gtk.Arguments.Gtk_Args   ) 
     return Boolean is 
   begin
      Do_Exit;
      return True;
   end On_Root_Window_Delete_Event;


   procedure On_File_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      null;
   end On_File_Activate;

   procedure On_New_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      Do_New;
      Clear_Work_Space;
      Rebuild_Canvas(Graph_Window);
   end On_New_Activate;

   procedure On_Open_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      Do_Open;
   end On_Open_Activate;


   procedure On_Save_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      Do_Save;
   end On_Save_Activate;


   procedure On_Save_As_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      Do_Save_As;
   end On_Save_As_Activate;


   procedure On_Exit_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      Do_Exit;
   end On_Exit_Activate;

   procedure On_New_Task_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      if (Get_Number_Of_Elements(Sys.Processors)=0)
            then
         Show_Message_Box(Lb_Define_Processors_Before(Current_Language));
      else
         Gtk_New (Tasks_Parameters);
         Show_All (Tasks_Parameters);
      end if;
   end On_New_Task_Activate;

   procedure On_Update_Task_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      if (Get_Number_Of_Elements(Sys.Tasks)=0)
            then
         Show_Message_Box(Lb_Define_Tasks_Before(Current_Language));
      else
         Gtk_New (Update_Tasks_Set);
         Show_All (Update_Tasks_Set);
      end if;
   end On_Update_Task_Activate;

   procedure On_New_Processor_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      Gtk_New (Processors_Parameters);
      Show_All (Processors_Parameters);
   end On_New_Processor_Activate;

   procedure On_Update_Processor_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      if (Get_Number_Of_Elements(Sys.Processors)=0)
            then
         Show_Message_Box(Lb_Define_Processors_Before(Current_Language));
      else
         Gtk_New (Update_Processors_Set);
         Show_All (Update_Processors_Set);
      end if;
   end On_Update_Processor_Activate;

   procedure On_Tasks_List_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin

      if (Get_Number_Of_Elements(Sys.Tasks)=0)
            then
         Show_Message_Box(Lb_Define_Tasks_Before(Current_Language));
      else
         Gtk_New (Display_Tasks_Set);
         Show_All (Display_Tasks_Set);
      end if;
   end On_Tasks_List_Activate;


   procedure On_Processors_List_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      if (Get_Number_Of_Elements(Sys.Processors)=0)
            then
         Show_Message_Box(Lb_Define_Processors_Before(Current_Language));
      else
         Gtk_New (Display_Processors_Set);
         Show_All (Display_Processors_Set);
      end if;
   end On_Processors_List_Activate;

   procedure On_Resource_List_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      if (Get_Number_Of_Elements(Sys.Resources)=0) then
         Show_Message_Box(Lb_Define_Resources_Before(Current_Language));
      else
         Gtk_New (Display_Resources_Set);
         Show_All (Display_Resources_Set);
      end if;
   end On_Resource_List_Activate;

   procedure On_Message_List_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin

      if (Get_Number_Of_Elements(Sys.Messages)=0) then
         Show_Message_Box(Lb_Define_Messages_Before(Current_Language));
      else
         Gtk_New (Display_Messages_Set);
         Show_All (Display_Messages_Set);
      end if;
   end On_Message_List_Activate;

   procedure On_Buffer_List_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      if (Get_Number_Of_Elements(Sys.Buffers)=0) then
         Show_Message_Box(Lb_Define_Buffers_Before(Current_Language));
      else
         Gtk_New (Display_Buffers_Set);
         Show_All (Display_Buffers_Set);
      end if;
   end On_Buffer_List_Activate;

   procedure On_Tools_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      null;
   end On_Tools_Activate;


   procedure On_Clear_Work_Space_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      Clear_Work_Space;
   end On_Clear_Work_Space_Activate;


   procedure On_Draw_Scheduling_Graph_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 

   begin
      Schedule_And_Check_Temporal_Constraints;
   end On_Draw_Scheduling_Graph_Activate;


   procedure On_Options_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      Gtk_New(Options);
      Show_All(Options);
   end On_Options_Activate;


   procedure On_Check_Scheduling_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      Check_Scheduling;
   end On_Check_Scheduling_Activate;


   procedure On_Help_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      null;
   end On_Help_Activate;


   procedure On_Update_Resource_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      if (Get_Number_Of_Elements(Sys.Resources)=0) then
         Show_Message_Box(Lb_Define_Resources_Before(Current_Language));
      else
         Gtk_New(Update_Resources_Set);
         Show_All(Update_Resources_Set);
         On_Cpu_Change_Show;
      end if;
   end On_Update_Resource_Activate;

   procedure On_New_Resource_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      if (Get_Number_Of_Elements(Sys.Processors)=0)
            then
         Show_Message_Box(Lb_Define_Processors_Before(Current_Language));
      else
         if (Get_Number_Of_Elements(Sys.Tasks)=0)
               then
            Show_Message_Box(Lb_Define_Tasks_Before(Current_Language));
         else
            Gtk_New(Resource_Parameter);
            Show_All(Resource_Parameter);
         end if;
      end if;
   end On_New_Resource_Activate;

   procedure On_About_Cheddar_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 

   begin
      Show_Message_Box(Lb_About_Text(Current_Language));
   end On_About_Cheddar_Activate;



   procedure On_Manual_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin

      Gtk_New(Scrolling_Text,Lb_Manual(Current_Language));
      Set_Usize(Scrolling_Text,690,425);


      Write_Title_Small(Scrolling_Text,Manual.Ustrfilemenu);
      Write_Text(Scrolling_Text,Manual.Ustrfilemenucontent);

      Write_Title_Small(Scrolling_Text,Manual.Ustreditmenu);
      Write_Text(Scrolling_Text,Manual.Ustreditmenucontent);

      Write_Title_Small(Scrolling_Text,Manual.Ustrviewmenu);
      Write_Text(Scrolling_Text,Manual.Ustrviewmenucontent);

      Write_Title_Small(Scrolling_Text,Manual.Ustrtoolsmenu);
      Write_Text(Scrolling_Text,Manual.Ustrtoolsmenucontent);

      Write_Title_Small(Scrolling_Text,Manual.Ustrhelpmenu);
      Write_Text(Scrolling_Text,Manual.Ustrhelpmenucontent);

      Show_All(Scrolling_Text);
   end On_Manual_Activate;


   procedure On_Bibliography_References_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      Gtk_New(Scrolling_Text,Lb_Ref(Current_Language));
      Set_Usize(Scrolling_Text,435, 440);
      Write_Text(Scrolling_Text,References.Text);
      Show_All(Scrolling_Text);
   end On_Bibliography_References_Activate;


   procedure On_Delete_Task_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      if (Get_Number_Of_Elements(Sys.Tasks)=0)
            then
         Show_Message_Box(
            Lb_Define_Tasks_Before(Current_Language));
      else
         Gtk_New(Question, Lb_Delete_Task(Current_Language),
            Lb_Task_Name(Current_Language) & " : ",
            To_Unbounded_String(""),
            Delete_Task_Callback'access);
         Show_All(Question);
      end if;
   end On_Delete_Task_Activate;

   procedure On_Delete_Resource_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      if (Get_Number_Of_Elements(Sys.Resources)=0)
            then
         Show_Message_Box(
            Lb_Define_Resources_Before(Current_Language));
      else
         Gtk_New(Question,Lb_Delete_Resource(Current_Language),
            Lb_Resource_Name(Current_Language) & " : ",
            To_Unbounded_String(""),
            Delete_Resource_Callback'access);
         Show_All(Question);
      end if;
   end On_Delete_Resource_Activate;


   procedure On_Delete_Processor_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      if (Get_Number_Of_Elements(Sys.Processors)=0)
            then
         Show_Message_Box(Lb_Define_Processors_Before(Current_Language));
      else
         Gtk_New(Question, Lb_Delete_Processor(Current_Language),
            Lb_Processor_Name(Current_Language) & " : ",
            To_Unbounded_String(""),
            Delete_Processor_Callback'access);
         Show_All(Question);
      end if;

   end On_Delete_Processor_Activate;

   procedure On_New_Message_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      Gtk_New(Messages_Parameters, To_Unbounded_String("cheddar_window"));
   end On_New_Message_Activate;

   procedure On_Delete_Message_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      if (Get_Number_Of_Elements(Sys.Messages)=0)
            then
         Show_Message_Box(
            Lb_Define_Messages_Before(Current_Language));
      else
         Gtk_New(Question, Lb_Delete_Message(Current_Language),
            Lb_Message_Name(Current_Language) & " : ",
            To_Unbounded_String(""),
            Delete_Message_Callback'access);
         Show_All(Question);
      end if;
   end On_Delete_Message_Activate;

   procedure On_Update_Message_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      if (Get_Number_Of_Elements(Sys.Messages)=0)
            then
         Show_Message_Box(
            Lb_Define_Messages_Before(Current_Language));
      else
         Gtk_New(Update_Messages_Set);
         Show_All(Update_Messages_Set);
      end if;
   end On_Update_Message_Activate;

   procedure On_New_Buffer_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      Gtk_New(Buffers_Parameters, To_Unbounded_String("cheddar_window"));

   end On_New_Buffer_Activate;

   procedure On_Delete_Buffer_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      if (Get_Number_Of_Elements(Sys.Buffers)=0)
            then
         Show_Message_Box(
            Lb_Define_Buffers_Before(Current_Language));
      else
         Gtk_New(Question, Lb_Delete_Buffer(Current_Language),
            Lb_Buffer_Name(Current_Language) & " : ",
            To_Unbounded_String(""),
            Delete_Buffer_Callback'access);
         Show_All(Question);
      end if;
   end On_Delete_Buffer_Activate;

   procedure On_Update_Buffer_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      Gtk_New(Update_Buffers_Set);
      --      Rebuild_canvas(Graph_Window);
   end On_Update_Buffer_Activate;

   procedure On_Check_Buffer_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      Check_Buffer;
   end On_Check_Buffer_Activate;

   procedure On_Pixmap_Draw_Scheduling_Clicked (
         Object : access Gtk_Toolbar_Record'Class ) is 

   begin
      Schedule_And_Check_Temporal_Constraints;
   end On_Pixmap_Draw_Scheduling_Clicked;


   --------------------------------------------------
   --    Toolbar menu callbacks
   --------------------------------------------------


   procedure On_Pixmap_New_Clicked (
         Object : access Gtk_Toolbar_Record'Class ) is 
   begin
      Do_New;
      Clear_Work_Space;
      Rebuild_Canvas(Graph_Window);
   end On_Pixmap_New_Clicked;


   procedure On_Pixmap_Open_Clicked (
         Object : access Gtk_Toolbar_Record'Class ) is 
   begin
      Do_Open;
   end On_Pixmap_Open_Clicked;


   procedure On_Pixmap_Save_Clicked (
         Object : access Gtk_Toolbar_Record'Class ) is 
   begin
      Do_Save;
   end On_Pixmap_Save_Clicked;


   procedure On_Pixmap_Save_As_Clicked (
         Object : access Gtk_Toolbar_Record'Class ) is 
   begin
      Do_Save_As;
   end On_Pixmap_Save_As_Clicked;


   procedure On_Pixmap_Exit_Clicked (
         Object : access Gtk_Toolbar_Record'Class ) is 
   begin
      Do_Exit;
   end On_Pixmap_Exit_Clicked;


   procedure On_Pixmap_Clear_Work_Space_Clicked (
         Object : access Gtk_Toolbar_Record'Class ) is 
   begin
      Clear_Work_Space;
   end On_Pixmap_Clear_Work_Space_Clicked;


   procedure On_Pixmap_Dependencies_Clicked (
         Object : access Gtk_Toolbar_Record'Class ) is 
   begin
      Gtk_New(Graph_Window);
      Show(Graph_Window);
   end On_Pixmap_Dependencies_Clicked;


   procedure On_Pixmap_Check_Scheduling_Clicked (
         Object : access Gtk_Toolbar_Record'Class ) is 
   begin
      Check_Scheduling;
   end On_Pixmap_Check_Scheduling_Clicked;

   --
   --

   procedure On_Blocking_Time_Callback (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      null;
   end On_Blocking_Time_Callback;

   procedure On_Blocking_Time_Display_Callback (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      if (Get_Number_Of_Elements(Sys.Resources)=0)
            then
         Show_Message_Box(
            Lb_Define_Resources_Before(Current_Language));
      else
         Display_Blocking_Time_For_All_Tasks(Sys.Tasks, Root_Window, False);
      end if;
   end On_Blocking_Time_Display_Callback;

   procedure On_Blocking_Time_Update_Callback (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      if (Get_Number_Of_Elements(Sys.Resources)=0)
            then
         Show_Message_Box(
            Lb_Define_Resources_Before(Current_Language));
      else
         Display_Blocking_Time_For_All_Tasks(Sys.Tasks, Root_Window, True);
      end if;
   end On_Blocking_Time_Update_Callback;



   --------------------------------------------------------------------
   -- Precedencies tools : deadline, jitter and priority modifications
   --------------------------------------------------------------------
   --
   --

   procedure On_Chetto_Priority_Callback (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      null;
   end  On_Chetto_Priority_Callback;



   procedure On_Chetto_Priority_Display_Callback (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      if (Get_Number_Of_Elements(Sys.Tasks)=0)
            then
         Show_Message_Box(
            Lb_Define_Tasks_Before(Current_Language));

      else
         Chetto_Parameter_Modifications(Root_Window, False, True, False);
      end if;
   end  On_Chetto_Priority_Display_Callback;



   procedure On_Chetto_Priority_Update_Callback (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      if (Get_Number_Of_Elements(Sys.Tasks)=0)
            then
         Show_Message_Box(
            Lb_Define_Tasks_Before(Current_Language));

      else
         Chetto_Parameter_Modifications(Root_Window, False, True, True);
      end if;
   end  On_Chetto_Priority_Update_Callback;

   --
   --

   procedure On_Chetto_Deadline_Callback (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      null;
   end  On_Chetto_Deadline_Callback;



   procedure On_Chetto_Deadline_Display_Callback (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      if (Get_Number_Of_Elements(Sys.Tasks)=0)
            then
         Show_Message_Box(
            Lb_Define_Tasks_Before(Current_Language));

      else
         Chetto_Parameter_Modifications(Root_Window, True, False, False);
      end if;
   end  On_Chetto_Deadline_Display_Callback;



   procedure On_Chetto_Deadline_Update_Callback (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      if (Get_Number_Of_Elements(Sys.Tasks)=0)
            then
         Show_Message_Box(
            Lb_Define_Tasks_Before(Current_Language));

      else
         Chetto_Parameter_Modifications(Root_Window, True, False, True);
      end if;
   end  On_Chetto_Deadline_Update_Callback;

   --
   --

   procedure On_Jitter_From_Response_Time_Callback (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      null;
   end  On_Jitter_From_Response_Time_Callback;

   procedure On_Jitter_From_Response_Time_Display_Callback (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      if (Get_Number_Of_Elements(Sys.Tasks)=0)
            then
         Show_Message_Box(
            Lb_Define_Tasks_Before(Current_Language));

      else
         Compute_End_To_End_Response_Time(Root_Window, False, False);
      end if;

   end  On_Jitter_From_Response_Time_Display_Callback;

   procedure On_Jitter_From_Response_Time_Update_Callback (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin

      if (Get_Number_Of_Elements(Sys.Tasks)=0)
            then
         Show_Message_Box(
            Lb_Define_Tasks_Before(Current_Language));

      else
         Compute_End_To_End_Response_Time(Root_Window, True, False);
      end if;


   end  On_Jitter_From_Response_Time_Update_Callback;


   procedure On_Jitter_From_Response_Time_Display_One_Step_Callback (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      if (Get_Number_Of_Elements(Sys.Tasks)=0)
            then
         Show_Message_Box(
            Lb_Define_Tasks_Before(Current_Language));

      else
         Compute_End_To_End_Response_Time(Root_Window, False, True);
      end if;


   end  On_Jitter_From_Response_Time_Display_One_Step_Callback;

   procedure On_Jitter_From_Response_Time_Update_One_Step_Callback (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin

      if (Get_Number_Of_Elements(Sys.Tasks)=0)
            then
         Show_Message_Box(
            Lb_Define_Tasks_Before(Current_Language));


      else
         Compute_End_To_End_Response_Time(Root_Window, True, True);
      end if;

   end  On_Jitter_From_Response_Time_Update_One_Step_Callback;



   -------------------------------------------------------------
   -- Set priorities according to a fixed priority scheduler
   -------------------------------------------------------------


   function Rm_Priorities_Callback (
         Value : Unbounded_String ) 
     return Boolean is 

      My_Iterator : Processors_Iterator;  
      A_Processor : Processor_Ptr;  

   begin

      loop

         Current_Element(Sys.Processors, A_Processor, My_Iterator);

         if A_Processor.Name = Value
               then
            Set_Priorities(Root_Window, Rate_Monotonic, Value);

            return True;
         end if;

         exit when Is_Last_Element(Sys.Processors, My_Iterator) = True;
         Next_Element(Sys.Processors, My_Iterator);

      end loop;


      Show_Message_Box(Lb_Processor_Not_Found(Current_Language));
      return False;

   end Rm_Priorities_Callback;



   function Dm_Priorities_Callback (
         Value : Unbounded_String ) 
     return Boolean is 

      My_Iterator : Processors_Iterator;  
      A_Processor : Processor_Ptr;  

   begin

      loop

         Current_Element(Sys.Processors, A_Processor, My_Iterator);

         if A_Processor.Name = Value
               then
            Set_Priorities(Root_Window, Deadline_Monotonic, Value);

            return True;
         end if;

         exit when Is_Last_Element(Sys.Processors, My_Iterator) = True;
         Next_Element(Sys.Processors, My_Iterator);

      end loop;


      Show_Message_Box(Lb_Processor_Not_Found(Current_Language));
      return False;

   end Dm_Priorities_Callback;


   procedure On_Dm_Priorities_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 

   begin

      if (Get_Number_Of_Elements(Sys.Tasks)=0)
            then
         Show_Message_Box(
            Lb_Define_Tasks_Before(Current_Language));
      else
         Gtk_New(Question, Lb_Set_Priorities_Dm(Current_Language),
            Lb_Processor_Name(Current_Language) & " : ",
            To_Unbounded_String(""),
            Dm_Priorities_Callback'access);
         Show_All(Question);
      end if;

   end On_Dm_Priorities_Activate;




   procedure On_Rm_Priorities_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin

      if (Get_Number_Of_Elements(Sys.Tasks)=0)
            then
         Show_Message_Box(
            Lb_Define_Tasks_Before(Current_Language));
      else
         Gtk_New(Question, Lb_Set_Priorities_Rm(Current_Language),
            Lb_Processor_Name(Current_Language) & " : ",
            To_Unbounded_String(""),
            Rm_Priorities_Callback'access);
         Show_All(Question);
      end if;

   end On_Rm_Priorities_Activate;


   procedure On_Duplicate_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 

   begin
      Gtk_New(Duplicate_Pkg.Duplicate);
      Show_All(Duplicate_Pkg.Duplicate);
   end On_Duplicate_Activate;



end Root_Window_Pkg.Callbacks;





