------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a real time simulator mainly designed for educational purpose. 
-- This program provides services to automaticly check temporal constraints 
-- of real time tasks.
--
-- Copyright (C) 2002   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the LIMI Team/EA 2215, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------



with Glib;
use Glib;

with Ihm_Tools;
use Ihm_Tools;

with Text_Io;
use Text_Io;

with Translate;
use Translate;

with Config;
use Config;

with Offsets;
use Offsets;
with Unbounded_Strings;
use Unbounded_Strings;
with Project;
use Project;

with Tasks;
use Tasks;
with Task_Set;
use Task_Set;
use Task_Set.Generic_Task_Set;
with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;

with Processors;
use Processors;
with Processor_Set;
use Processor_Set;
use Processor_Set.Generic_Processor_Set;
with Gtk.Radio_Menu_Item;
use Gtk.Radio_Menu_Item;
with Graph_Window_Pkg;
use Graph_Window_Pkg;

package body Update_Tasks_Set_Pkg.Callbacks is

   procedure Show_Tasks (
         A_Task_Name : Unbounded_String ) is 
      A_Task          : Task_Ptr;  
      My_Iterator     : Tasks_Iterator;  
      Line            : Gint;  
      Task_Share      : Chars_Ptr_Array (1 .. 9);  
      Capacity        : Unbounded_String;  
      Start_Time      : Unbounded_String;  
      Jitter          : Unbounded_String;  
      Deadline        : Unbounded_String;  
      Period          : Unbounded_String;  
      Priority        : Unbounded_String;  
      Blocking_Time   : Unbounded_String;  
      A_Periodic_Task : Periodic_Task_Ptr;  

   begin
      Clear(Update_Tasks_Set.Tasks_List);
      Reset_Iterator(My_Iterator);
      loop
         Current_Element(Sys.Tasks, A_Task, My_Iterator);
         Capacity   := To_Unbounded_String(Natural'Image(A_Task.Capacity));
         Deadline   := To_Unbounded_String(Natural'Image(A_Task.Deadline));
         Start_Time := To_Unbounded_String(Natural'Image(A_Task.
               Start_Time));
         Blocking_Time := To_Unbounded_String(Natural'Image(A_Task.
               Blocking_Time));
         Priority   := To_Unbounded_String(Natural'Image(Natural(A_Task.
                  Priority)));
         if(A_Task.Task_Type=Periodic_Type) then
            A_Periodic_Task := Periodic_Task_Ptr(A_Task);
            Jitter   := To_Unbounded_String(Natural'Image(A_Periodic_Task.
                  Jitter));
            Period  := To_Unbounded_String(Natural'Image(A_Periodic_Task.
                  Period));

         else
            Jitter   := To_Unbounded_String(" 0");
            Period  := To_Unbounded_String(" 0");

         end if;

         --Create the line
         Task_Share := To_String(A_Task.Name)+To_String(A_Task.Location)+
            To_String(Capacity)+To_String(Jitter)+To_String(Deadline)+
            To_String(Period)+To_String(Start_Time)+To_String(Priority)+
            To_String(Blocking_Time);
         --Display the Line
         Line := Append(Update_Tasks_Set.Tasks_List, Task_Share);

         exit when Is_Last_Element(Sys.Tasks, My_Iterator) = True;

         Next_Element(Sys.Tasks, My_Iterator);

      end loop;

   end Show_Tasks;


   procedure On_Click_List (
         List : access Gtk_Clist_Record'Class ) is 


      Name          : Unbounded_String;  
      Capacity      : Unbounded_String;  
      Jitter        : Unbounded_String;  
      Deadline      : Unbounded_String;  
      Period        : Unbounded_String;  
      Start_Time    : Unbounded_String;  
      Priority      : Unbounded_String;  
      Blocking_Time : Unbounded_String;  
      Row           : Gint;  
      My_Iterator   : Processor_Set.Generic_Processor_Set.Iterator;  
      A_Processor   : Processor_Ptr;  
      Index         : Strings_Table_Range                          := Strings_Table_Range'First;
         

      A_Task         : Task_Ptr;  
      Current        : Gint;  
      Task_Str       : Unbounded_String;  
      Location_Items : Strings_Table;  
      Current_Policy : Gint;  


   begin
      Row:=Get_Focus_Row(Update_Tasks_Set.Tasks_List);
      Name := To_Unbounded_String(Get_Text(Update_Tasks_Set.Tasks_List,
            Row,0));
      Update_Tasks_Set.Current_Task := Name;
      Capacity      := To_Unbounded_String(Get_Text(Update_Tasks_Set.
            Tasks_List,Row,2));
      Jitter        := To_Unbounded_String(Get_Text(Update_Tasks_Set.
            Tasks_List,Row,3));
      Deadline      := To_Unbounded_String(Get_Text(Update_Tasks_Set.
            Tasks_List,Row,4));
      Period        := To_Unbounded_String(Get_Text(Update_Tasks_Set.
            Tasks_List,Row,5));
      Start_Time    := To_Unbounded_String(Get_Text(Update_Tasks_Set.
            Tasks_List,Row,6));
      Priority      := To_Unbounded_String(Get_Text(Update_Tasks_Set.
            Tasks_List,Row,7));
      Priority      := To_Unbounded_String(Get_Text(Update_Tasks_Set.
            Tasks_List,Row,7));
      Blocking_Time := To_Unbounded_String(Get_Text(Update_Tasks_Set.
            Tasks_List,Row,8));

      Gtk.Gentry.Set_Text(Update_Tasks_Set.Name, To_String(Name));
      Gtk.Gentry.Set_Text(Update_Tasks_Set.Capacity, To_String(Capacity));
      Gtk.Gentry.Set_Text(Update_Tasks_Set.Jitter, To_String(Jitter));
      Gtk.Gentry.Set_Text(Update_Tasks_Set.Deadline, To_String(Deadline));
      Gtk.Gentry.Set_Text(Update_Tasks_Set.Period, To_String(Period));
      Gtk.Gentry.Set_Text(Update_Tasks_Set.Start_Time, To_String(
            Start_Time));
      Gtk.Gentry.Set_Text(Update_Tasks_Set.Priority, To_String(Priority));
      Gtk.Gentry.Set_Text(Update_Tasks_Set.Blocking_Time, To_String(
            Blocking_Time));

      Task_Str:=Update_Tasks_Set.Current_Task;
      if (Task_Str /= To_Unbounded_String("")) then
         A_Task:=Search_Task(Sys.Tasks,Task_Str);
         Update_Tasks_Set.Current_Task_Ptr := A_Task;

         Gtk.Gentry.Set_Text(Update_Tasks_Set.Offset_Act1, A_Task.Offset(
               1).Activation'Img);
         Gtk.Gentry.Set_Text(Update_Tasks_Set.Offset_Act2, A_Task.Offset(
               2).Activation'Img);
         Gtk.Gentry.Set_Text(Update_Tasks_Set.Offset_Act3, A_Task.Offset(
               3).Activation'Img);
         Gtk.Gentry.Set_Text(Update_Tasks_Set.Offset_Val1, A_Task.Offset(
               1).Value'Img);
         Gtk.Gentry.Set_Text(Update_Tasks_Set.Offset_Val2, A_Task.Offset(
               2).Value'Img);
         Gtk.Gentry.Set_Text(Update_Tasks_Set.Offset_Val3, A_Task.Offset(
               3).Value'Img);

         ----- Rebuild The CPU Option Menu & select the right item
         --
         Initialize(Location_Items);
         Index:=0;
         Processor_Set.Generic_Processor_Set.Reset_Iterator(My_Iterator);

         loop
            Current_Element(Sys.Processors, A_Processor, My_Iterator);
            Location_Items(Index):=A_Processor.Name;
            if A_Processor.Name = A_Task.Location then
               Current:=Gint(Index);
            end if;
            Index:=Index+1;
            exit when Is_Last_Element(Sys.Processors, My_Iterator) = True;
            Next_Element(Sys.Processors, My_Iterator);
         end loop;
         Destroy(Update_Tasks_Set.Processor_Menu);
         Gtk.Option_Menu.Gtk_New(Update_Tasks_Set.Processor_Menu);
         Update_Tasks_Set.Clist_Processor_Menu:= Widget_Slist.Null_List;

         Build_Option_Menu (Update_Tasks_Set.Processor_Menu,
            Update_Tasks_Set.Clist_Processor_Menu,
            Location_Items, Current, null);
         Attach_Defaults(Update_Tasks_Set.Task_Table, Update_Tasks_Set.
            Processor_Menu,1,2,1,2);
         Show_All(Update_Tasks_Set);
         ----- End Rebuild
         --


         -- rebuild policy menu
         --
         Initialize(Location_Items);
         Location_Items(0):=To_Unbounded_String("SCHED_FIFO");
         Location_Items(1):=To_Unbounded_String("SCHED_RR");
         Location_Items(2):=To_Unbounded_String("SCHED_OTHERS");

         if (A_Task.Policy = Sched_Fifo)
               then
            Current_Policy:=0;
         else
            if (A_Task.Policy = Sched_Rr)
                  then
               Current_Policy:=1;
            else
               Current_Policy:=2;
            end if;
         end if;

         Destroy(Update_Tasks_Set.Policy_Menu);
         Gtk.Option_Menu.Gtk_New(Update_Tasks_Set.Policy_Menu);
         Update_Tasks_Set.Clist_Policy_Menu:= Widget_Slist.Null_List;

         Build_Option_Menu (Update_Tasks_Set.Policy_Menu, Update_Tasks_Set.
            Clist_Policy_Menu, Location_Items, Current_Policy, null);
         Attach_Defaults(Update_Tasks_Set.Task_Table, Update_Tasks_Set.
            Policy_Menu,1,2,8,9);
         Show_All(Update_Tasks_Set);
         -- end rebuild
         --


         Update_Tasks_Set.Current_Task := A_Task.Name;

      end if;


   end On_Click_List;


   procedure On_Ok_Pressed (
         Object : access Gtk_Button_Record'Class ) is 
   begin

      Is_New:=False;
      Is_Saved:=False;

      Rebuild_Canvas(Graph_Window);
      Hide(Update_Tasks_Set);
   end On_Ok_Pressed;


   procedure On_Cancel_Pressed (
         Object : access Gtk_Button_Record'Class ) is 
   begin
      Sys.Tasks:=Update_Tasks_Set.Undo_Task;
      Destroy(Update_Tasks_Set);
   end On_Cancel_Pressed;

   procedure On_Modify_Pressed (
         Object : access Gtk_Button_Record'Class ) is 
      Ok                        : Boolean;  
      Task_Name                 : Unbounded_String;  
      Task_Processor            : Unbounded_String;  
      Task_Capacity             : Integer;  
      Task_Jitter               : Integer;  
      Task_Deadline             : Integer;  
      Task_Period               : Integer;  
      Task_Start_Time           : Integer;  
      Task_Blocking_Time        : Integer;  
      Task_Priority             : Priority_Range;  
      Task_Policy               : Policies;  
      Policy_Index              : Gint;  
      Task_Priority2            : Integer;  
      Task_Offset               : Offset_Table;  
      Task_Capacity_String      : Unbounded_String;  
      Task_Jitter_String        : Unbounded_String;  
      Task_Deadline_String      : Unbounded_String;  
      Task_Period_String        : Unbounded_String;  
      Task_Start_Time_String    : Unbounded_String;  
      Task_Blocking_Time_String : Unbounded_String;  
      Task_Priority_String      : Unbounded_String;  
      A_Task                    : Task_Ptr;  
      Cpu_Index                 : Gint;  
      A_Processor               : Processor_Ptr;  
      My_Iterator               : Task_Set.Generic_Task_Set.Iterator;  
      Current                   : Unbounded_String;  
   begin
      Task_Name := To_Unbounded_String(Get_Text(Update_Tasks_Set.Name));
      Reset_Iterator(My_Iterator);
      loop
         Current_Element(Sys.Tasks, A_Task, My_Iterator);
         if (Task_Name = A_Task.Name) then
            if(A_Task.Name/=Update_Tasks_Set.Current_Task) then
               Show_Message_Box(Lb_Task_Name(Current_Language) &
                  Lb_Already_Defined(Current_Language));
               Sys.Tasks:=Update_Tasks_Set.Undo_Task;
               return;
            end if;
         end if;
         exit when Is_Last_Element(Sys.Tasks, My_Iterator) = True;
         Next_Element(Sys.Tasks, My_Iterator);
      end loop;

      Task_Capacity_String := To_Unbounded_String(Get_Text(
            Update_Tasks_Set.Capacity));
      Task_Jitter_String := To_Unbounded_String(Get_Text(Update_Tasks_Set.
            Jitter));
      Task_Deadline_String :=  To_Unbounded_String(Get_Text(
            Update_Tasks_Set.Deadline));
      Task_Period_String := To_Unbounded_String(Get_Text(Update_Tasks_Set.
            Period));
      Task_Start_Time_String := To_Unbounded_String(Get_Text(
            Update_Tasks_Set.Start_Time));
      Task_Blocking_Time_String := To_Unbounded_String(Get_Text(
            Update_Tasks_Set.Blocking_Time));
      Task_Priority_String := To_Unbounded_String(Get_Text(
            Update_Tasks_Set.Priority));

      Cpu_Index:= Gint'Val (Gint(Get_Number_Of_Elements(Sys.Processors)) -
         Gint(Selected_Button (Update_Tasks_Set.Clist_Processor_Menu)));
      Cpu_Index:=Cpu_Index-1;

      Get_Element_Number(Sys.Processors,A_Processor,Processors_Range(
            Cpu_Index));
      Task_Processor :=A_Processor.Name;


      Policy_Index:= Gint(Selected_Button (Update_Tasks_Set.
            Clist_Policy_Menu));

      if (Policy_Index = 2)
            then
         Task_Policy:=Sched_Fifo;
      else
         if (Policy_Index = 1)
               then
            Task_Policy:=Sched_Rr;
         else
            Task_Policy:=Sched_Others;
         end if;
      end if;


      To_Integer(Task_Capacity_String,Task_Capacity, Ok);
      if not Ok
            then
         Show_Message_Box(Lb_Capacity(Current_Language) &
            Lb_Must_Be_Numeric(Current_Language));
         Sys.Tasks:=Update_Tasks_Set.Undo_Task;
         return;
      end if;

      To_Integer(Task_Jitter_String,Task_Jitter, Ok);
      if not Ok
            then
         Show_Message_Box(Lb_Jitter(Current_Language) & Lb_Must_Be_Numeric(
               Current_Language));
         Sys.Tasks:=Update_Tasks_Set.Undo_Task;
         return;
      end if;

      To_Integer(Task_Deadline_String,Task_Deadline, Ok);
      if not Ok
            then
         Show_Message_Box(Lb_Deadline(Current_Language) &
            Lb_Must_Be_Numeric(Current_Language));
         Sys.Tasks:=Update_Tasks_Set.Undo_Task;
         return;
      end if;

      To_Integer(Task_Period_String,Task_Period, Ok);
      if not Ok
            then
         Show_Message_Box(Lb_Period(Current_Language) & Lb_Must_Be_Numeric(
               Current_Language));
         Sys.Tasks:=Update_Tasks_Set.Undo_Task;
         return;
      end if;

      To_Integer(Task_Start_Time_String,Task_Start_Time, Ok);
      if not Ok
            then
         Show_Message_Box(Lb_Start_Time(Current_Language) &
            Lb_Must_Be_Numeric(Current_Language));
         Sys.Tasks:=Update_Tasks_Set.Undo_Task;
         return;
      end if;

      To_Integer(Task_Blocking_Time_String,Task_Blocking_Time, Ok);
      if not Ok
            then
         Show_Message_Box(Lb_Blocking_Time(Current_Language) &
            Lb_Must_Be_Numeric(Current_Language));
         Sys.Tasks:=Update_Tasks_Set.Undo_Task;
         return;
      end if;

      To_Integer(Task_Priority_String,Task_Priority2, Ok);
      if not Ok
            then
         Show_Message_Box(Lb_Priority(Current_Language) &
            Lb_Must_Be_Numeric(Current_Language));
         Sys.Tasks:=Update_Tasks_Set.Undo_Task;
         return;
      end if;

      --
      -- Offset information
      --
      Current:=To_Unbounded_String(Get_Text(Update_Tasks_Set.Offset_Val1));
      if(Current="")
            then
         Task_Offset(1).Value:=0;
      else
         To_Integer(Current, Task_Offset(1).Value,Ok);
         if not Ok
               then
            Show_Message_Box(To_Unbounded_String("Offset") &
               Lb_Must_Be_Numeric(Current_Language) &
               To_Unbounded_String("Offset(1)") & Lb_Value(
                  Current_Language));
            Sys.Tasks:=Update_Tasks_Set.Undo_Task;
            return;
         end if;
      end if;
      Current:=To_Unbounded_String(Get_Text(Update_Tasks_Set.Offset_Val2));
      if(Current="")
            then
         Task_Offset(2).Value:=0;
      else
         To_Integer(Current, Task_Offset(2).Value,Ok);
         if not Ok
               then
            Show_Message_Box(To_Unbounded_String("Offset") &
               Lb_Must_Be_Numeric(Current_Language) &
               To_Unbounded_String("Offset(2) ") & Lb_Value(
                  Current_Language));
            Sys.Tasks:=Update_Tasks_Set.Undo_Task;
            return;
         end if;
      end if;
      Current:=To_Unbounded_String(Get_Text(Update_Tasks_Set.Offset_Val3));
      if(Current="")
            then
         Task_Offset(3).Value:=0;
      else
         To_Integer(Current, Task_Offset(3).Value,Ok);
         if not Ok
               then
            Show_Message_Box(To_Unbounded_String("Offset") &
               Lb_Must_Be_Numeric(Current_Language) &
               To_Unbounded_String("Offset(3) ") & Lb_Value(
                  Current_Language));
            Sys.Tasks:=Update_Tasks_Set.Undo_Task;
            return;
         end if;
      end if;
      Current:=To_Unbounded_String(Get_Text(Update_Tasks_Set.Offset_Act1));
      if(Current="")
            then
         Task_Offset(1).Activation:=0;
      else
         To_Integer(Current, Task_Offset(1).Activation,Ok);
         if not Ok
               then
            Show_Message_Box(To_Unbounded_String("Offset") &
               Lb_Must_Be_Numeric(Current_Language) &
               To_Unbounded_String("Offset(1) ") & Lb_Activation(
                  Current_Language));
            Sys.Tasks:=Update_Tasks_Set.Undo_Task;
            return;
         end if;
      end if;
      Current:=To_Unbounded_String(Get_Text(Update_Tasks_Set.Offset_Act2));
      if(Current="")
            then
         Task_Offset(2).Activation:=0;
      else
         To_Integer(Current, Task_Offset(2).Activation,Ok);
         if not Ok
               then
            Show_Message_Box(To_Unbounded_String("Offset") &
               Lb_Must_Be_Numeric(Current_Language) &
               To_Unbounded_String("Offset(2) ")& Lb_Activation(
                  Current_Language));
            Sys.Tasks:=Update_Tasks_Set.Undo_Task;
            return;
         end if;
      end if;
      Current:=To_Unbounded_String(Get_Text(Update_Tasks_Set.Offset_Act3));
      if(Current="")
            then
         Task_Offset(3).Activation:=0;
      else
         To_Integer(Current, Task_Offset(3).Activation,Ok);
         if not Ok
               then
            Show_Message_Box(To_Unbounded_String("Offset") &
               Lb_Must_Be_Numeric(Current_Language) &
               To_Unbounded_String("Offset(3) ")& Lb_Activation(
                  Current_Language));
            Sys.Tasks:=Update_Tasks_Set.Undo_Task;
            return;
         end if;
      end if;

      if (Update_Tasks_Set.Current_Task_Ptr.Offset(1).Activation<-0
            or Update_Tasks_Set.Current_Task_Ptr.Offset(2).Activation<0
            or Update_Tasks_Set.Current_Task_Ptr.Offset(3).Activation<0) then
         Show_Message_Box(Lb_Offset_Val(Current_Language) & Lb_Mandatory(
               Current_Language));
         Sys.Tasks:=Update_Tasks_Set.Undo_Task;
         return;
      end if;

      if (Update_Tasks_Set.Current_Task_Ptr.Offset(1).Value<0
            or Update_Tasks_Set.Current_Task_Ptr.Offset(2).Value<0
            or Update_Tasks_Set.Current_Task_Ptr.Offset(3).Value<0) then
         Show_Message_Box(Lb_Offset_Act(Current_Language) & Lb_Mandatory(
               Current_Language));
         Sys.Tasks:=Update_Tasks_Set.Undo_Task;
         return;
      end if;

      if (Task_Capacity<=0)
            then
         Show_Message_Box(Lb_Capacity(Current_Language) & Lb_Must_Be(
               Current_Language) & ">0");
         Sys.Tasks:=Update_Tasks_Set.Undo_Task;
         return;
      end if;

      if (Task_Jitter<0)
            then
         Show_Message_Box(Lb_Jitter(Current_Language) & Lb_Must_Be(
               Current_Language) & "=>0");
         Sys.Tasks:=Update_Tasks_Set.Undo_Task;
         return;
      end if;

      if (Task_Period<0)
            then
         Show_Message_Box(Lb_Period(Current_Language) & Lb_Must_Be(
               Current_Language) & "=>0");
         return;
      end if;

      if (Task_Deadline<Task_Jitter)
            then
         Show_Message_Box(Lb_Deadline(Current_Language) & Lb_Must_Be(
               Current_Language) & Lb_Superior(Current_Language) &
            Lb_Jitter(Current_Language));
         return;
      end if;

      if (Task_Deadline<0)
            then
         Show_Message_Box(Lb_Deadline(Current_Language) & Lb_Must_Be(
               Current_Language) & ">0");
         Sys.Tasks:=Update_Tasks_Set.Undo_Task;
         return;
      end if;

      if (Task_Start_Time<0)
            then
         Show_Message_Box(Lb_Start_Time(Current_Language) & Lb_Must_Be(
               Current_Language) & ">0");
         Sys.Tasks:=Update_Tasks_Set.Undo_Task;
         return;
      end if;

      if (Task_Blocking_Time<0)
            then
         Show_Message_Box(Lb_Blocking_Time(Current_Language) & Lb_Must_Be(
               Current_Language) & ">0");
         Sys.Tasks:=Update_Tasks_Set.Undo_Task;
         return;
      end if;

      if ((Natural(Task_Priority2)<Natural(Priority_Range'First))
            or (Natural(Task_Priority2)>Natural(Priority_Range'Last)) )
            then
         Show_Message_Box(Lb_Invalid_Priority(Current_Language));
         Sys.Tasks:=Update_Tasks_Set.Undo_Task;
         return;
      end if;

      Task_Priority := Priority_Range(Task_Priority2);


      if (Task_Priority2 /= 0) and
            (Task_Policy=Sched_Others)
            then
         Show_Message_Box(Lb_Policy_Control(Current_Language) );
         return;
      end if;

      if (Task_Priority2 = 0) and
            (Task_Policy/=Sched_Others)
            then
         Show_Message_Box(Lb_Policy_Control(Current_Language) );
         return;
      end if;


      Reset_Iterator(My_Iterator);
      loop

         Current_Element(Sys.Tasks, A_Task, My_Iterator);
         if A_Task.Name = Update_Tasks_Set.Current_Task then

            Delete(Sys.Tasks, A_Task);

            Add_Task (Sys.Tasks, Task_Name, Task_Processor, Natural(
                  Task_Start_Time), Natural(Task_Capacity), Natural(
                  Task_Period), Natural(Task_Deadline), Natural(
                  Task_Jitter), Natural(Task_Blocking_Time), Task_Priority,
               Task_Policy, Task_Offset);


            Update_Tasks_Set.Current_Task := Task_Name;
            exit;
         end if;


         exit when Is_Last_Element(Sys.Tasks, My_Iterator) = True;
         Next_Element(Sys.Tasks, My_Iterator);

      end loop;



      Show_Tasks(A_Task.Name);

   end On_Modify_Pressed;

end Update_Tasks_Set_Pkg.Callbacks;
