------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a real time simulator mainly designed for educational purpose. 
-- This program provides services to automaticly check temporal constraints 
-- of real time tasks.
--
-- Copyright (C) 2002   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the LIMI Team/EA 2215, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------



with Glib;
use Glib;
with Gtk;
use Gtk;
with Gtk.Widget;
use Gtk.Widget;
with Gtk.Enums;
use Gtk.Enums;

with Ihm_Tools;
use Ihm_Tools;
with Callbacks_Cheddar;
use Callbacks_Cheddar;
with Update_Tasks_Set_Pkg.Callbacks;
use Update_Tasks_Set_Pkg.Callbacks;

with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;
with Unbounded_Strings;
use  Unbounded_Strings;
with Project;
use Project;
with Tasks;
use Tasks;
with Task_Set;
use Task_Set;
use Task_Set.Generic_Task_Set;
with Translate;
use Translate;
with Config;
use Config;

package body Update_Tasks_Set_Pkg is

   procedure Gtk_New (
         Update_Tasks_Set :    out Update_Tasks_Set_Access ) is 
   begin
      Update_Tasks_Set := new Update_Tasks_Set_Record;
      Update_Tasks_Set_Pkg.Initialize (Update_Tasks_Set);
   end Gtk_New;

   procedure Initialize (
         Update_Tasks_Set : access Update_Tasks_Set_Record'Class ) is 


      A_Task          : Task_Ptr;  
      My_Iterator     : Tasks_Iterator;  
      Items           : Strings_Table;  
      A_Line          : Chars_Ptr_Array (1 .. 9);  
      Line            : Gint;  
      A_Periodic_Task : Periodic_Task_Ptr;  
      Capacity        : Unbounded_String;  
      Start_Time      : Unbounded_String;  
      Jitter          : Unbounded_String;  
      Deadline        : Unbounded_String;  
      Period          : Unbounded_String;  
      Priority        : Unbounded_String;  
      Blocking_Time   : Unbounded_String;  

   begin
      Gtk.Window.Initialize(Update_Tasks_Set, Window_Toplevel);
      Set_Policy (Update_Tasks_Set, True, True, True);
      Set_Title (Update_Tasks_Set, To_String(Lb_Update_Task(
               Current_Language)));
      Set_Position (Update_Tasks_Set, Win_Pos_Center_Always);
      Set_Modal (Update_Tasks_Set, True);
      Grab_Focus(Update_Tasks_Set);

      Gtk.Box.Gtk_New_Vbox(Update_Tasks_Set.Vbox_Main, False, 10);
      Set_Border_Width (Update_Tasks_Set.Vbox_Main, 10);
      Gtk.Box.Gtk_New_Hbox(Update_Tasks_Set.Hbox_Task, False, 10);
      Gtk.Box.Gtk_New_Hbox(Update_Tasks_Set.Hbox_Confirm, False, 10);
      Gtk.Box.Gtk_New_Vbox(Update_Tasks_Set.Vbox_List, False, 10);
      Gtk.Box.Gtk_New_Hbox(Update_Tasks_Set.Hbox_Modify, False, 10);
      Gtk.Box.Gtk_New_Vbox(Update_Tasks_Set.Vbox_Update, False, 10);

      Add(Update_Tasks_Set, Update_Tasks_Set.Vbox_Main);
      Add(Update_Tasks_Set.Vbox_Main, Update_Tasks_Set.Hbox_Task);
      Add(Update_Tasks_Set.Hbox_Task, Update_Tasks_Set.Vbox_List);
      Add(Update_Tasks_Set.Hbox_Task, Update_Tasks_Set.Vbox_Update);
      Add(Update_Tasks_Set.Vbox_Main, Update_Tasks_Set.Hbox_Confirm);

      Update_Tasks_Set.Undo_Task:= Sys.Tasks;

      Reset_Iterator(My_Iterator);

      Gtk.Frame.Gtk_New(Update_Tasks_Set.Frame_Task,"");

      Gtk.Table.Gtk_New (Update_Tasks_Set.Task_Table,2, 15, False);
      Set_Border_Width (Update_Tasks_Set.Task_Table, 5);

      Add(Update_Tasks_Set.Vbox_Update, Update_Tasks_Set.Frame_Task);
      Add(Update_Tasks_Set.Frame_Task, Update_Tasks_Set.Task_Table);

      Gtk.Option_Menu.Gtk_New(Update_Tasks_Set.Processor_Menu);
      Build_Option_Menu (Update_Tasks_Set.Processor_Menu,
         Update_Tasks_Set.Clist_Processor_Menu, Items, -1, null);

      Gtk.Option_Menu.Gtk_New(Update_Tasks_Set.Policy_Menu);
      Build_Option_Menu (Update_Tasks_Set.Policy_Menu,
         Update_Tasks_Set.Clist_Policy_Menu, Items, -1, null);

      Gtk.Label.Gtk_New(Update_Tasks_Set.Name_Label, To_String(Lb_Name(
               Current_Language)) & To_String(Lb_Colon));
      Gtk.Label.Gtk_New(Update_Tasks_Set.Processor_Label,To_String(
            Lb_Processor(Current_Language)) & To_String(Lb_Colon));
      Gtk.Label.Gtk_New(Update_Tasks_Set.Capacity_Label,To_String(
            Lb_Capacity(Current_Language)) & To_String(Lb_Colon));
      Gtk.Label.Gtk_New(Update_Tasks_Set.Jitter_Label,To_String(Lb_Jitter(
               Current_Language)) & To_String(Lb_Colon));
      Gtk.Label.Gtk_New(Update_Tasks_Set.Deadline_Label, To_String(
            Lb_Deadline(Current_Language)) & To_String(Lb_Colon));
      Gtk.Label.Gtk_New(Update_Tasks_Set.Period_Label, To_String(
            Lb_Period(Current_Language)) & To_String(Lb_Colon));
      Gtk.Label.Gtk_New(Update_Tasks_Set.Start_Time_Label, To_String(
            Lb_Start_Time(Current_Language)) & To_String(Lb_Colon));
      Gtk.Label.Gtk_New(Update_Tasks_Set.Priority_Label, To_String(
            Lb_Priority(Current_Language)) & To_String(Lb_Colon));
      Gtk.Label.Gtk_New(Update_Tasks_Set.Policy_Label, To_String(
            Lb_Policy(Current_Language)) & To_String(Lb_Colon));
      Gtk.Label.Gtk_New(Update_Tasks_Set.Blocking_Time_Label, To_String(
            Lb_Blocking_Time(Current_Language)) & To_String(Lb_Colon));
      Gtk.Label.Gtk_New(Update_Tasks_Set.Blocking_Time_Label, To_String(
            Lb_Blocking_Time(Current_Language)) & To_String(Lb_Colon));
      Gtk.Label.Gtk_New(Update_Tasks_Set.Offset_Label, To_String(
            Lb_Offsets(Current_Language)) & To_String(Lb_Colon));
      Gtk.Label.Gtk_New(Update_Tasks_Set.Offset_Act_Label, To_String(
            Lb_Activations(Current_Language)) & To_String(Lb_Colon));
      Gtk.Label.Gtk_New(Update_Tasks_Set.Offset_Val_Label, To_String(
            Lb_Values(Current_Language)) & To_String(Lb_Colon));

      Gtk.Gentry.Gtk_New(Update_Tasks_Set.Name, 50);
      Set_Editable (Update_Tasks_Set.Name, False);

      Gtk.Gentry.Gtk_New(Update_Tasks_Set.Start_Time, 50);
      Gtk.Gentry.Gtk_New(Update_Tasks_Set.Capacity, 50);
      Gtk.Gentry.Gtk_New(Update_Tasks_Set.Period, 50);
      Gtk.Gentry.Gtk_New(Update_Tasks_Set.Deadline, 50);
      Gtk.Gentry.Gtk_New(Update_Tasks_Set.Jitter, 50);
      Gtk.Gentry.Gtk_New(Update_Tasks_Set.Priority, 50);
      Gtk.Gentry.Gtk_New(Update_Tasks_Set.Blocking_Time, 50);
      Gtk.Gentry.Gtk_New(Update_Tasks_Set.Offset_Val1, 50);
      Gtk.Gentry.Gtk_New(Update_Tasks_Set.Offset_Val2, 50);
      Gtk.Gentry.Gtk_New(Update_Tasks_Set.Offset_Val3, 50);
      Gtk.Gentry.Gtk_New(Update_Tasks_Set.Offset_Act1, 50);
      Gtk.Gentry.Gtk_New(Update_Tasks_Set.Offset_Act2, 50);
      Gtk.Gentry.Gtk_New(Update_Tasks_Set.Offset_Act3, 50);

      Gtk_New_Hseparator (Update_Tasks_Set.Hseparator1);
      Gtk_New_Vseparator (Update_Tasks_Set.Vseparator);
      Gtk_New_Vseparator (Update_Tasks_Set.Vseparator2);


      Attach_Defaults(Update_Tasks_Set.Task_Table, Update_Tasks_Set.
         Name_Label, 0, 1, 0, 1);
      Attach_Defaults(Update_Tasks_Set.Task_Table, Update_Tasks_Set.
         Processor_Label, 0, 1, 1, 2);
      Attach_Defaults(Update_Tasks_Set.Task_Table, Update_Tasks_Set.
         Capacity_Label, 0, 1, 2, 3);
      Attach_Defaults(Update_Tasks_Set.Task_Table, Update_Tasks_Set.
         Jitter_Label, 0, 1, 3, 4);
      Attach_Defaults(Update_Tasks_Set.Task_Table, Update_Tasks_Set.
         Deadline_Label, 0, 1, 4, 5);
      Attach_Defaults(Update_Tasks_Set.Task_Table, Update_Tasks_Set.
         Period_Label, 0, 1, 5, 6);
      Attach_Defaults(Update_Tasks_Set.Task_Table, Update_Tasks_Set.
         Start_Time_Label, 0, 1, 6, 7);
      Attach_Defaults(Update_Tasks_Set.Task_Table, Update_Tasks_Set.
         Priority_Label, 0, 1, 7, 8);
      Attach_Defaults(Update_Tasks_Set.Task_Table, Update_Tasks_Set.
         Policy_Label, 0, 1, 8, 9);
      Attach_Defaults(Update_Tasks_Set.Task_Table, Update_Tasks_Set.
         Blocking_Time_Label, 0, 1, 9, 10);
      Attach_Defaults(Update_Tasks_Set.Task_Table, Update_Tasks_Set.
         Hseparator1, 0, 2, 10, 11);
      Attach_Defaults(Update_Tasks_Set.Task_Table, Update_Tasks_Set.
         Offset_Label, 0, 1, 11, 12);
      Attach_Defaults(Update_Tasks_Set.Task_Table, Update_Tasks_Set.
         Offset_Act_Label, 0, 1, 12, 13);
      Attach_Defaults(Update_Tasks_Set.Task_Table, Update_Tasks_Set.
         Offset_Val_Label, 1, 2, 12, 13);


      Attach_Defaults(Update_Tasks_Set.Task_Table, Update_Tasks_Set.Name,
         1, 2, 0, 1);
      Attach_Defaults(Update_Tasks_Set.Task_Table, Update_Tasks_Set.
         Processor_Menu, 1, 2, 1, 2);
      Attach_Defaults(Update_Tasks_Set.Task_Table, Update_Tasks_Set.
         Capacity, 1, 2, 2, 3);
      Attach_Defaults(Update_Tasks_Set.Task_Table, Update_Tasks_Set.
         Jitter, 1, 2, 3, 4);
      Attach_Defaults(Update_Tasks_Set.Task_Table, Update_Tasks_Set.
         Deadline, 1, 2, 4, 5);
      Attach_Defaults(Update_Tasks_Set.Task_Table, Update_Tasks_Set.
         Period, 1, 2, 5, 6);
      Attach_Defaults(Update_Tasks_Set.Task_Table, Update_Tasks_Set.
         Start_Time, 1, 2, 6, 7);
      Attach_Defaults(Update_Tasks_Set.Task_Table, Update_Tasks_Set.
         Priority, 1, 2, 7, 8);
      Attach_Defaults(Update_Tasks_Set.Task_Table, Update_Tasks_Set.
         Policy_Menu, 1, 2, 8, 9);
      Attach_Defaults(Update_Tasks_Set.Task_Table, Update_Tasks_Set.
         Blocking_Time, 1, 2, 9, 10);
      Attach_Defaults(Update_Tasks_Set.Task_Table, Update_Tasks_Set.
         Offset_Act1, 0, 1, 13, 14);
      Attach_Defaults(Update_Tasks_Set.Task_Table, Update_Tasks_Set.
         Offset_Act2, 0, 1, 14, 15);
      Attach_Defaults(Update_Tasks_Set.Task_Table, Update_Tasks_Set.
         Offset_Act3, 0, 1, 15, 16);
      Attach_Defaults(Update_Tasks_Set.Task_Table, Update_Tasks_Set.
         Offset_Val1, 1, 2, 13, 14);
      Attach_Defaults(Update_Tasks_Set.Task_Table, Update_Tasks_Set.
         Offset_Val2, 1, 2, 14, 15);
      Attach_Defaults(Update_Tasks_Set.Task_Table, Update_Tasks_Set.
         Offset_Val3, 1, 2, 15, 16);


      Gtk.Scrolled_Window.Gtk_New(Update_Tasks_Set.Scroll_List, null, null);
      Add(Update_Tasks_Set.Vbox_List,Update_Tasks_Set.Scroll_List);

      Update_Tasks_Set.List_Titles:= To_String(Lb_Name(Current_Language))
         +To_String(Lb_Processor(Current_Language))
         +To_String(Lb_Capacity(Current_Language))
         +To_String(Lb_Jitter(Current_Language))
         +To_String(Lb_Deadline(Current_Language))
         +To_String(Lb_Period(Current_Language))
         +To_String(Lb_Start_Time(Current_Language))
         +To_String(Lb_Priority(Current_Language))
         +To_String(Lb_Blocking_Time(Current_Language));

      Gtk.Clist.Gtk_New(Update_Tasks_Set.Tasks_List, 9, Update_Tasks_Set.
         List_Titles);
      Set_Usize(Update_Tasks_Set.Tasks_List,475,200);
      Add(Update_Tasks_Set.Scroll_List,Update_Tasks_Set.Tasks_List);
      Reset_Iterator(My_Iterator);

      loop

         Current_Element(Sys.Tasks, A_Task, My_Iterator);
         Capacity      := To_Unbounded_String(Natural'Image(A_Task.
               Capacity));
         Deadline      := To_Unbounded_String(Natural'Image(A_Task.
               Deadline));
         Start_Time    := To_Unbounded_String(Natural'Image(A_Task.
               Start_Time));
         Priority      := To_Unbounded_String(Natural'Image(Natural(
                  A_Task.Priority)));
         Blocking_Time := To_Unbounded_String(Natural'Image(A_Task.
               Blocking_Time));
         if(A_Task.Task_Type=Periodic_Type) then
            A_Periodic_Task := Periodic_Task_Ptr(A_Task);
            Jitter   := To_Unbounded_String(Natural'Image(A_Periodic_Task.
                  Jitter));
            Period  := To_Unbounded_String(Natural'Image(A_Periodic_Task.
                  Period));

         else
            Jitter:=To_Unbounded_String(" 0");
            Period:=To_Unbounded_String(" 0");
         end if;

         -- Create the line
         --
         A_Line := To_String(A_Task.Name)+To_String(A_Task.Location)+
            To_String(Capacity)+To_String(Jitter)+To_String(Deadline)+
            To_String(Period)+To_String(Start_Time)+To_String(Priority)+
            To_String(Blocking_Time);
         -- Display the Line
         --
         Line := Append(Update_Tasks_Set.Tasks_List, A_Line);

         exit when Is_Last_Element(Sys.Tasks, My_Iterator) = True;

         Next_Element(Sys.Tasks, My_Iterator);

      end loop;
      Add(Update_Tasks_Set.Vbox_List, Update_Tasks_Set.Hbox_Modify);


      Gtk.Button.Gtk_New(Update_Tasks_Set.Ok_Button, "Ok");
      Gtk.Button.Gtk_New(Update_Tasks_Set.Cancel_Button, To_String(
            Lb_Cancel(Current_Language)));
      Add(Update_Tasks_Set.Hbox_Modify, Update_Tasks_Set.Ok_Button);
      Add(Update_Tasks_Set.Hbox_Modify, Update_Tasks_Set.Cancel_Button);
      Add(Update_Tasks_Set.Hbox_Modify, Update_Tasks_Set.Vseparator2);
      Gtk.Button.Gtk_New(Update_Tasks_Set.Button_Modify, To_String(
            Lb_Modify(Current_Language)));
      Add(Update_Tasks_Set.Hbox_Modify, Update_Tasks_Set.Button_Modify);

      List_Callback.Connect (Update_Tasks_Set.Tasks_List, "select_row",
         List_Callback.To_Marshaller (On_Click_List'access));
      Button_Callback.Connect (Update_Tasks_Set.Button_Modify, "clicked",
         Button_Callback.To_Marshaller (On_Modify_Pressed'access));
      Button_Callback.Connect (Update_Tasks_Set.Cancel_Button, "clicked",
         Button_Callback.To_Marshaller (On_Cancel_Pressed'access));

      Button_Callback.Connect (Update_Tasks_Set.Ok_Button, "clicked",
         Button_Callback.To_Marshaller (On_Ok_Pressed'access));

   end Initialize;

end Update_Tasks_Set_Pkg;
