------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a real time simulator mainly designed for educational purpose. 
-- This program provides services to automaticly check temporal constraints 
-- of real time tasks.
--
-- Copyright (C) 2002   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the LIMI Team/EA 2215, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------



with Ada.Io_Exceptions;
use  Ada.Io_Exceptions;

with Unbounded_Strings;
use  Unbounded_Strings;

with Systems;

with Gnat.Command_Line;
use Gnat.Command_Line;

with  Gnat.Os_Lib;
use Gnat.Os_Lib;

with Text_Io;
use Text_Io;


with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;

with Systems;
use Systems;


procedure Dump_Sys is 



   Sys       : System;  
   File_Name : Unbounded_String;  



   procedure Copyright is 
   begin
      Put_Line(
         "Dump_sys version 1.1, Copyright (C) 2002, F. Singhoff ");
      Put_Line(
         "Dump_sys is developped by the LIMI Team/EA 2215, University of Brest");
      Put_Line("Dump_sys comes with ABSOLUTELY NO WARRANTY; ");
      Put_Line(
         "This is free software, and you are welcome to redistribute it");
      Put_Line(
         "under certain conditions; See the GNU General Public License for details.");
   end Copyright;


   procedure Usage is 
   begin
      Put_Line (
         "Dump_sys allows you to display in a textual manner the content of a Cheddar project file.");

      Copyright;
      New_Line;
      Put_Line(
         "See Cheddar home page :  http://beru.univ-brest.fr/~singhoff/cheddar ");
      New_Line;
      New_Line;
      Put_Line("Usage : dump_sys project-name");
      New_Line;
      New_Line;
   end Usage;




begin


   -- Get arguments 
   loop
      case Getopt ("u") is
         when Ascii.Nul =>
            exit;

         when 'u' =>
            Usage;
            Os_Exit(0);

         when others =>
            Usage;
            Os_Exit(0);
      end case;
   end loop;

   loop
      declare
         S : constant String := Get_Argument (Do_Expansion => True);  

      begin
         exit when S'Length = 0;
         File_Name := File_Name & S;
      end;
   end loop;


   if(File_Name = Empty_String)
         then
      Usage;
      Os_Exit(0);

   else
      Reset(Sys);

      Read_From_File(Sys, File_Name);

      Dump(Sys);

   end if;



exception
   when Invalid_Switch    =>
      begin
         Put_Line ("dump_sys.adb : invalid Switch " & Full_Switch);
         Usage;
      end;
   when Invalid_Parameter =>
      Put_Line ("dump_sys.adb : missing parameter for switch " &
         Full_Switch);
   when Ada.Io_Exceptions.Name_Error =>
      Put_Line("dump_sys.adb : Can not read project files");



end  Dump_Sys;



