------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a real time simulator mainly designed for educational purpose.
-- This program provides services to automaticly check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the LIMI Team/EA 2215, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------




with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;


with Task_Set;
use Task_Set;
with Scheduler;
use Scheduler;

with Scheduling_Sequences;
use Scheduling_Sequences;

with Text_Io;
use Text_Io;

with Buffers;
use Buffers;
use Buffers.Buffer_Roles;

with Buffer_Set;
use Buffer_Set;
use Buffer_Set.Generic_Buffer_Set;



with Tasks;
use Tasks;

with Time_Units;
use Time_Units;
use Time_Units.A_Time_Unit;

with Systems;
use Systems;

with Processors;
use Processors;

with Processor_Set;
use Processor_Set;
use Processor_Set.Generic_Processor_Set;


procedure Test_Delay_Buffer is 

   Sched           : Scheduling_Sequence;  
   My_Size         : Buffer_Size_Table;  
   Bound           : Natural;  
   Sys             : System;  
   A_Processor     : Processor_Ptr;  
   A_Buffer        : Buffer_Ptr;  
   My_Iterator     : Processors_Iterator;  
   My_Iterator_Buf : Buffers_Iterator;  

begin

   Reset(Sys);

   -- read the project created with cheddar
   Read_From_File(Sys, To_Unbounded_String("sys.cheddar"));
   Dump(Sys.Tasks);

   -- build scheduling sequence and check buffer size
   -- for each processor of the readed cheddar project
   Reset_Iterator(My_Iterator);
   loop
      Initialize(My_Size);
      Current_Element(Sys.Processors, A_Processor, My_Iterator);

      -- build scheduling sequence
      Build_Scheduling_Sequence(A_Processor.Scheduler.all,
         Sys.Tasks, A_Processor.Name,
         Sched,20); -- changer le 20 !!!
      Dump(Sched);

      Reset_Iterator(My_Iterator_Buf);
      loop
         Current_Element(Sys.Buffers, A_Buffer, My_Iterator_Buf);

         -- compute the maximum occupation of for the buffer
         Get_Buffer_Size( Sched, A_Buffer.Roles, Sys.Tasks, My_Size);
         Bound:=Get_Max_Size(My_Size);

         -- write it
         Put_Line ("max buffer size = " & Bound'Img);
         New_Line;
         for I in 0 .. Sched.Scheduled.Nb_Entries-1 loop
            Put_Line("Time = " & My_Size(I).Time'Img & "/Size = " &
               My_Size(I).
               Size'Img);
         end loop;

         -- Next buffer
         exit when Is_Last_Element(Sys.Buffers, My_Iterator_Buf) = True;
         Next_Element(Sys.Buffers, My_Iterator_Buf);
      end loop;

      -- next processor
      exit when Is_Last_Element(Sys.Processors, My_Iterator) = True;
      Next_Element(Sys.Processors, My_Iterator);
   end loop;
end Test_Delay_Buffer;

