------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a real time simulator mainly designed for educational purpose. 
-- This program provides services to automaticly check temporal constraints 
-- of real time tasks.
--
-- Copyright (C) 2002   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the LIMI Team/EA 2215, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------




with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;

with Io_Definitions;
use Io_Definitions;

with tasks;
use tasks;
with Task_Set;
use Task_Set;

with Dependencies;
use Dependencies;
with Messages;
use Messages;
with Message_Set;
use Message_Set;

with Task_Dependencies;
use Task_Dependencies;
use Task_Dependencies.Half_Dep_Set;

with Offsets;
use Offsets;

procedure Write_Depend is 


   Msg    : Messages_Set;  
   Msg1,  
   Msg2   : Message_Ptr;  
   Tasks1 : Tasks_Set;  
   Dep1   : Tasks_Dependencies;  
   Offset : Offset_Table;  
   Cpu    : Unbounded_String   := To_Unbounded_String ("cpu");  

begin


   Initialize(Offset);
   Reset(Dep1);

   Add_Task(Tasks1, To_Unbounded_String("T1"), Cpu, 0, 5, 12, 0, 0, 0, 1,
      sched_fifo, Offset);
   Add_Task(Tasks1, To_Unbounded_String("T2"), Cpu, 0, 2, 6,  0, 0, 0, 1,
      sched_fifo, Offset);
   Add_Task(Tasks1, To_Unbounded_String("T3"), Cpu, 0, 5, 24, 0, 0, 0, 1,
      sched_fifo, Offset);
   Add_Task(Tasks1, To_Unbounded_String("T4"), Cpu, 0, 5, 24, 0, 0, 0, 1,
      sched_fifo, Offset);

   Add_Message(Msg, Msg1, To_Unbounded_String("m1"), 1000, 20, 0, 0);
   Add_Message(Msg, Msg2, To_Unbounded_String("m2"), 1000, 10, 0, 0);

   Dump(Tasks1);
   Dump(Msg);

   Dump(Dep1);

   Add_Tasks_Dependencies(Dep1, Tasks1,
      To_Unbounded_String("T1"), To_Unbounded_String("T2"), Dependency_Ptr(
         Msg1));

   Add_Tasks_Dependencies(Dep1, Tasks1,
      To_Unbounded_String("T1"), To_Unbounded_String("T3"), Dependency_Ptr(
         Msg1));

   Add_Tasks_Dependencies(Dep1, Tasks1,
      To_Unbounded_String("T3"), To_Unbounded_String("T4"), Dependency_Ptr(
         Msg2));

   Dump(Dep1);

   Write_To_File(Dep1, To_Unbounded_String("depend.cheddar"),
      Save_In_New_File);

   Delete_Tasks_Dependencies(Dep1,
      To_Unbounded_String("T1"), To_Unbounded_String("T2"));

   Dump(Dep1);


end  Write_Depend;

