------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a real time simulator mainly designed for educational purpose. 
-- This program provides services to automaticly check temporal constraints 
-- of real time tasks.
--
-- Copyright (C) 2002   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the LIMI Team/EA 2215, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------






generic
   type Element is private; 
   type Index is private; 

   Size  : Natural := 100;  
   First : Natural := 0;  
   with procedure Put (
         E : in     Element ); 
   with procedure Initialize (
         E :    out Element ); 
   with procedure Put (
         E : in     Index ); 

package Tables is


   -- Do not change range 0..first+size
   -- intro first..first+size : it's a tips to
   -- avoid constraint error with nb_entries
   -- define by a table_range (and not a natural)
   --
   type Table_Range is new Natural range 0 .. First + Size; 
   First_Range : Table_Range :=Table_Range(First);


   type Item is 
      record 
         Data : Element;  
         Item : Index;  
      end record; 

   type Data_Table is array (Table_Range) of Item; 

   type Table is 
      record 
         Nb_Entries : Table_Range := 0;  
         Entries    : Data_Table;  
      end record; 


   procedure Dump (
         A_Item : in     Item ); 

   procedure Dump (
         A_Table : in     Table ); 

   procedure Initialize (
         A_Table :    out Table ); 


   procedure Delete (
         A_Table : in out Table;      
         A_Item  :        Table_Range ); 

   procedure Add (
         A_Table   : in out Table;  
         A_Index   :        Index;  
         A_Element :        Element ); 



   pragma Inline(Add);
   pragma Inline(Delete);

end Tables;



