------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a real time simulator mainly designed for educational purpose. 
-- This program provides services to automaticly check temporal constraints 
-- of real time tasks.
--
-- Copyright (C) 2002   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the LIMI Team/EA 2215, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------




with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;


package Unbounded_Strings is

   Max_Strings : constant Natural := 100;  

   Unbounded_Sp : constant Unbounded_String := To_Unbounded_String (" ");
      

   Unbounded_Lf : constant Unbounded_String := To_Unbounded_String ("" &
     Ascii.Lf);  
   Unbounded_Ht : constant Unbounded_String := To_Unbounded_String ("" &
     Ascii.Ht);  
   Empty_String : constant Unbounded_String := To_Unbounded_String ("");
      



   type Strings_Table_Range is new Natural range 0 .. Max_Strings; 
   type Strings_Table is array (Strings_Table_Range) of Unbounded_String; 

   procedure Put (
         Str : in     Unbounded_String ); 
   procedure Put_Line (
         Str : in     Unbounded_String ); 


   procedure To_Natural (
         From : in     Unbounded_String; 
         To   :    out Natural;          
         Ok   :    out Boolean           ); 

   procedure To_Integer (
         From : in     Unbounded_String; 
         To   :    out Integer;          
         Ok   :    out Boolean           ); 

   pragma Inline (To_Natural);
   pragma Inline (To_Integer);

   procedure Initialize (
         St :    out Unbounded_String ); 
   procedure Initialize (
         St :    out Strings_Table ); 

   pragma Inline (Initialize);

end Unbounded_Strings;
